#========================================================================================
# package      DE_EPAGES::Content::Hooks::Shop
# state        public
# description  create default shop
#========================================================================================
package DE_EPAGES::Content::Hooks::Shop;

use strict;
use DE_EPAGES::Content::API::Cartridge;
use DE_EPAGES::Object::API::Factory qw ( LoadObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw (ImportObjects);

#========================================================================================
# function     OnInsertShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnInsertShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Imports default pages (StartPage, Imprint, TermsAndConditions,
#               CustomerInformationshop) into a new shop.
#               (Hook OBJ_InsertShop)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                   <li>Object - shop - object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};
    my $Cartridge = DE_EPAGES::Content::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/DefaultShop.xml', $Shop->pathString);
}

#========================================================================================
# function     OnAfterUpdateShop
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateShop( $hParams );
#----------------------------------------------------------------------------------------
# description  if shop is opened, then update template types
#               (Hook OBJ_AfterUpdateShop)
#----------------------------------------------------------------------------------------
# input        $hParams | primary key of tax class, key is
#               <ul>
#                 <li>Object        - shop object          - object
#                 <li>ObjectID      - shop object id            - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateShop {
    my ($hParams) = @_;

    if ((exists $hParams->{'Values'}->{'ClosedByProvider'}) or
        (exists $hParams->{'Values'}->{'ClosedByProviderTemporarily'})){
        $hParams->{'Object'}->child('Categories')->correctTemplateType() if $hParams->{'Object'}->get('ClosedOrClosedTempByProvider') == 0;
    }

    # delete previous imported legal pages for unity because we store legal pages only in unity
    if ((exists $hParams->{'Values'}->{'IsUnityRegistered'}) and defined $hParams->{'Values'}->{'IsUnityRegistered'}){
        my $Shop = $hParams->{'Object'};
        foreach my $Child (qw(Imprint TermsAndConditions PrivacyPolicy CustomerInformation ShippingAndPaymentInformation )){
            $Shop->folder('Categories')->child($Child)->delete() if $Shop->folder('Categories')->existsChild($Child);
        }
    }

    return;
}

#========================================================================================
# function     OnBeforeCreateUnityShop
#----------------------------------------------------------------------------------------
# syntax       OnBeforeCreateUnityShop( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_BeforeCreateUnityShop.
#               Initializes the fist steps.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>Object  - shop                              - object
#                 <li>Values  - new shop values                   - ref.hash
#                 <li>Content - content to patch to the unity API - ref.hash
#               </ul>
#========================================================================================
sub OnBeforeCreateUnityShop {
    my ($hParams) = @_;
    my $Shop = $hParams->{'Object'};

    # initialize first steps
    my $hItems = $Shop->get('NextSteps') // {};
    $hItems->{'LegalPages'} = 0;
    $Shop->set({'NextSteps' => $hItems});

    return;
}

#========================================================================================
# function     OnInsertShopFeature
#----------------------------------------------------------------------------------------
# syntax       OnInsertShopFeature( $hParams );
#----------------------------------------------------------------------------------------
# description  Triggered by the hook API_InsertShopFeature.
#               Imports default legal pages (Imprint, TermsAndConditions, PrivacyPolicy
#               CustomerInformations, ShippingInformation) into a new Unity shop
#               if the feature 'Unity' is set.
#----------------------------------------------------------------------------------------
# input        $hParams | hook parameters
#               <ul>
#                 <li>ShopID     - shop id    - int
#                 <li>FeatureID  - feature id - int
#               </ul>
#========================================================================================
sub OnInsertShopFeature {
    my ($hParams) = @_;
    my $ShopID = $hParams->{'ShopID'};
    my $FeatureID = $hParams->{'FeatureID'};
    my $Feature = LoadObject ( $FeatureID );
    if ($Feature->alias eq 'Unity') {
        my $Shop = LoadObject( $ShopID );
        my $Unity = $Shop->featureMaxValue('Unity');
        my $Cartridge = DE_EPAGES::Content::API::Cartridge->new;
        ImportObjects($Cartridge->xmlDirectory().'/DefaultShopUnity.xml', $Shop->pathString) if $Unity;
    }
}

#========================================================================================
# function     OnAfterUpdateShopFeature
#----------------------------------------------------------------------------------------
# syntax       OnAfterUpdateShopFeature( $hParams );
#----------------------------------------------------------------------------------------
# description  OnAfterUpdateShopFeature is triggered,
#               initializes the legal pages for unity shops
#----------------------------------------------------------------------------------------
# input        $hShopFeature - feature object, used keys are
#               <ul>
#                   <li>FeatureID - feature id            | integer</li>
#                   <li>ShopID    - shop id               | integer</li>
#                   <li>MaxValue  - maximum feature value | int</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnAfterUpdateShopFeature {
    my ($hShopFeature) = @_;

    my $Feature = LoadObject($hShopFeature->{'FeatureID'});
    return unless ($Feature->alias eq 'Unity' and $hShopFeature->{'MaxValue'});

    my $Shop = LoadObject($hShopFeature->{'ShopID'});
    my $Cartridge = DE_EPAGES::Content::API::Cartridge->new;
    ImportObjects($Cartridge->xmlDirectory().'/DefaultShopUnity.xml', $Shop->pathString) unless $Shop->existsChild('LegalPagesUnity');
    return;
}

1;
