#========================================================================================
# package      DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal
# state        public
# base         DE_EPAGES::Order::API::Object::LineItemPayment
# description  object interface for LineItemPaymentExternal.
#========================================================================================
package DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal;
use base DE_EPAGES::Order::API::Object::LineItemPayment;

use strict;
use DE_EPAGES::Core::API::Error qw (Error);
use DE_EPAGES::Core::API::PerlTools qw (fcmp);
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $LineItemPaymentExternal->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'LineItemPaymentExternal' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $LineItemPaymentExternal->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentExternal = DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal->insert($hInfo);
# example      my $Class = LoadClassByAlias('LineItemPaymentExternal');
#               my $LineItemPaymentExternal = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertLineItemPaymentExternal' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $LineItemPaymentExternal | new object | object
# hook         OBJ_InsertLineItemPaymentExternal | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentExternalID - line item payment external id - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    my $self = $class->SUPER::insert($hInfo);

    TriggerHook('OBJ_InsertLineItemPaymentExternal', {'Object' => $self, 'LineItemPaymentExternalID' => $self->id });
    delete $hInfo->{$_} foreach qw ( CurrencyID Amount TransID TransTime TransTimeMod TransStatus AuthCode ReturnCode TransID2 UserID UserIP InternalMethod Message AvsCode IsSecure Field1 Field2 Field3 ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentExternal->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteLineItemPaymentExternal' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteLineItemPaymentExternal | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentExternalID - line item payment external id - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    TriggerHook('OBJ_DeleteLineItemPaymentExternal', {'Object' => $self, 'LineItemPaymentExternalID' => $self->id });
    $self->SUPER::delete;
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentExternal->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateLineItemPaymentExternal' and 'OBJ_AfterUpdateLineItemPaymentExternal'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateLineItemPaymentExternal | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentExternalID - line item payment external id - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateLineItemPaymentExternal | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>LineItemPaymentExternalID - line item payment external id - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateLineItemPaymentExternal', {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentExternalID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateLineItemPaymentExternal',  {'Object'=>$self, 'Values'=> $hValues, 'LineItemPaymentExternalID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

#========================================================================================
# function     executePayment
# state        public
#----------------------------------------------------------------------------------------
# syntax       $PaymentLineItem->executePayment($hValues);
#----------------------------------------------------------------------------------------
# description  This function allows a payment line item to initiate the payment process.
#               Override this function to interrupt the basket processing, for example to
#               display additional pages for collection payment information.
#               The default implementation does nothing.
#               Called by <function DE_EPAGES::Order::Hooks::Basket::OnBasket2OrderExecutePayment>
#               Called by <function DE_EPAGES::Order::Hooks::Order::OnOrderExecutePayment>
#----------------------------------------------------------------------------------------
# input        $hValues | environment information
#                       <ul>
#                           <li>Basket - basket object (optional)</li>
#                           <li>Servlet - servlet handling the current request</li>
#                           <li>Order - order object (optional)</li>
#                       </ul>
#                   | ref.hash
#========================================================================================
sub executePayment{
    my $self = shift;
    my ($hParams)= @_;

    $self->updateOnExecute($hParams);

    my $Object = $hParams->{'Order'} // $hParams->{'Basket'};

    # show payment template
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError'    => 0,
            'ViewObjectID' => $Object->id,
        },
        'ViewAction' => 'ViewPayment' . $self->get('PaymentType')->alias,
    });

    return;
}

#========================================================================================
# function     updateOnExecute
# state        public
#----------------------------------------------------------------------------------------
# syntax       $updateOnExecute($hParams);
#----------------------------------------------------------------------------------------
# description  This functions sets amount and currency.
#----------------------------------------------------------------------------------------
# input        $hValues | environment information
#                       <ul>
#                           <li>Basket - basket object</li>
#                           <li>Servlet - servlet handling the current request</li>
#                           <li>Order - order object (optional)</li>
#                       </ul>
#                   | ref.hash
#========================================================================================
sub updateOnExecute {
    my $self = shift;
    my ($hParams)= @_;

    my $Container = $self->container;

    #set the amount for payment
    my $amount     = $Container->get('GrandTotal');
    my $CurrencyID = $Container->get('CurrencyID');

    if (!defined($amount)) {
        $amount = 0;
    }
    else {
        my $LocaleID = $Container->get('LocaleID');
        my $Locale   = $Container->getSite()->siteLocale($LocaleID);
        $amount      = $Locale->roundMoney($amount, $CurrencyID)
    }

    my $PaymentMethod = $self->get('PaymentMethod');

    $self->set({
        'Amount'            => $amount,
        'CurrencyID'        => $CurrencyID,
        'IsTestTransaction' => $PaymentMethod->get('IsInTestMode')
    });

    return;
}

#========================================================================================
# function     checkAmount
# state        public
#----------------------------------------------------------------------------------------
# syntax       $result = $PaymentLineItem->checkAmount;
#----------------------------------------------------------------------------------------
# description  This functions allows a payment line item to check if the ordered amount
#               differs from the amount defined in the transaction.
#               Overwrite this function for security checks in external payment or
#               checkout methods.
#----------------------------------------------------------------------------------------
# return       $result | true if the ordered amount matches the transasctional amount
#                       | boolean
#========================================================================================
sub checkAmount {
    my $self = shift;

    return $self->get('CurrencyID') eq $self->parent->get('CurrencyID')  &&
           fcmp($self->get('Amount'), $self->parent->get('GrandTotal')) == 0;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::ExternalPayment::API::Object::LineItemPaymentExternal->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;

    my $amount = $Parent->get('GrandTotal');
    my $CurrencyID = $Parent->get('CurrencyID');
    if (!defined($amount)) {
        $amount = 0;
    }
    else {
        my $LocaleID = $Parent->get('LocaleID');
        my $Locale = $Parent->getSite()->siteLocale($LocaleID);
        $amount = $Locale->roundMoney($amount, $CurrencyID)
    }

    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'Amount' => $amount,
        'CurrencyID' => $CurrencyID,
        'IsSecure' => 0,
        'LineItemPrice' => 0,
    };
}

#========================================================================================
# function     getDisplayTransID
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DisplayTransID = $LineItemPayment->getDisplayTransID() ;
#----------------------------------------------------------------------------------------
# description  The Payment transaction have two possible ID's:
#                1. TransID   main transaction ID
#                2. TransID2  extra ID ( some cartridges could work with the main ID and
#                             this secondary ID for presentation purposes )
#               This method is a suitable way to return the ID to display in templates.
#               By default we return the main ID ( TransID ) but a specialized cartridge
#               could overwrite this method to return TransID2 ( or whatever ! )
#               The attribute 'DisplayTransID' in the class 'LineItemPaymentExternal'
#               calls this method to return the presentation ID to any template which uses
#               the attribute #DisplayTransID ( see template 'ContentLineLineItemPayment'
#               in the 'BasePageType' )
#----------------------------------------------------------------------------------------
# return       $DisplayTransID | TransID attribute | string
#========================================================================================
sub getDisplayTransID {
    my $self = shift;

    return $self->get('TransID') ;
}


#========================================================================================
# function     getAdditionalData
# state        public
#----------------------------------------------------------------------------------------
# syntax       $LineItemPaymentPayPalPlus->getAdditionalData( );
#----------------------------------------------------------------------------------------
# description  it provides null additional payment data as default for external payments
#               that don't have this method call implemented as part of the Order Rest
#               API call.
#----------------------------------------------------------------------------------------
# return       $hAdditionalInfo | hash with payment information | hash.ref
#========================================================================================
sub getAdditionalData {
    my $self = shift;

    return;
}

1;
