#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::MolliePaymentsService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::MolliePayments::API::MolliePaymentsService
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::MolliePaymentsService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled GetParametersFromMockifyCall );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;

use JSON::Any;
use Test::Exception;
use Test::Mockify::Matcher qw ( Object Any );
use Test::Mockify::Sut;
use Test::Mockify;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant ACCESS_TOKEN => 'FAKE_ACCESS_TOKEN';
use constant EXPECTED_SUCCESS_RESPONSE => {'response' => 'Very successful response'};

use constant EPAGESJ_SUFFIX_URL_PATH => '/epjsuffix/path';

use constant DECODED_JSON_RESPONSE_AS_HASH => { 'resfield1' => 'resvalue1', 'resfield2' => 'resvalue2' };

sub test {
    my $self = shift;

    $self->test_MolliePaymentsServiceTokensWithPost();
    $self->test_MolliePaymentsServiceMethodsWithGet();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_MolliePaymentsServiceTokensWithPost {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $MockedRequestWrapper = $self->_createRequestWrapper();
    my $mockedTokensRequest =$self->_createTokensRequest();
    my $SUT = $self->_createSUT($mockedTokensRequest, $MockedRequestWrapper);
    $SUT->withTestMode(1);
    my $ActualResponse = $SUT->doPostRequest();

    ok(WasCalled(
        $MockedRequestWrapper, 'doPost'),
            "$SubTestName - doPost in RequestWrapper was called."
    );

    is(
            $ActualResponse,
            EXPECTED_SUCCESS_RESPONSE,
            "$SubTestName - POST request response is correct."
    );

    my $aParameters = GetParametersFromMockifyCall($MockedRequestWrapper, 'doPost');

    is_deeply(
        $aParameters,
        [1],
        "$SubTestName - doPost in RequestWrapper was called with correct parameters."
    );

    ok(WasCalled(
        $mockedTokensRequest, 'handleResponse'),
        "$SubTestName - handleResponse in TokensRequest with Post was called."
    );


}

sub test_MolliePaymentsServiceMethodsWithGet {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $mockedTokensRequest = $self->_createTokensRequest();
    my $MockedRequestWrapper = $self->_createRequestWrapper();
    my $SUT = $self->_createSUT($mockedTokensRequest, $MockedRequestWrapper);

    my $ActualResponse = $SUT->doGetRequest();

    ok(WasCalled(
        $MockedRequestWrapper, 'doGet'),
        "$SubTestName - doGet in RequestWrapper was called."
    );

    is(
        $ActualResponse,
        EXPECTED_SUCCESS_RESPONSE,
        "$SubTestName - GET request response is correct."
    );

    my $aParameters = GetParametersFromMockifyCall($MockedRequestWrapper, 'doGet');

    is_deeply(
        $aParameters,
        [undef],
        "$SubTestName - doGet in RequestWrapper was called with correct parameters."
    );

    ok(WasCalled(
        $mockedTokensRequest, 'handleResponse'),
        "$SubTestName - handleResponse in TokensRequest with Post was called."
    );
}
#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _createPaymentMethod {
    my $self = shift;
    my ($hVars) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', {}
    );
    $Mockify->addMockWithReturnValue('getSite', $self->_createShop());

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;
    my $hAttributes = {
        'Alias' => 'MyShop',
        'ShopTypeRef' => $self->_createShopTypeRef()
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );
    return $Mockify->getMockObject();
}

sub _createRequestWrapper {
    my $self = shift;

    my $MockRequestWrapper = Test::Mockify->new('DE_EPAGES::MolliePayments::API::Requests::RequestWrapper');
    $MockRequestWrapper->mock('doPost')->when(Any())->thenReturn(EXPECTED_SUCCESS_RESPONSE);
    $MockRequestWrapper->mock('doGet')->when(Any())->thenReturn(EXPECTED_SUCCESS_RESPONSE);

    return $MockRequestWrapper->getMockObject();
}

sub _createSUT {
    my $self = shift;
    my ($Request, $MockedRequestWrapper) = @_;

    my $MockifySut = Test::Mockify::Sut->new(
            'DE_EPAGES::MolliePayments::API::MolliePaymentsService', [ $Request ]
    );

    $MockifySut->mock('_createRequestWrapper')
            ->when(Object())->thenReturn($MockedRequestWrapper);

    return $MockifySut->getMockObject();
}

sub _createTokensRequest {
    my $self = shift;

    my $PaymentMethod = $self->_createPaymentMethod();
    my $aParameters = [
        {
            'payment' => $PaymentMethod,
            'code' => 'MyAuthorizationCode'
        }
    ];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::MolliePayments::API::Requests::Tokens', $aParameters);

    $Mockify->addMockWithReturnValue('getShop', $self->_createShop());
    $Mockify->addMockWithReturnValueAndParameterCheck(
            'getEpagesjRequestSuffixUrlPath', EPAGESJ_SUFFIX_URL_PATH, ['string']
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
            'getHttpHeaders', {'Authorization' => 'Basic f3adfadsraradgfgh'} , ['string']
    );
    $Mockify->addMockWithReturnValueAndParameterCheck(
                'getPayload',
                {'reqfield1' => 'reqvalue1', 'reqfield2' => 'reqvalue2', 'reqfield3' => 'reqvalue3'} ,
                ['string']
    );
    $Mockify->addMockWithReturnValue('getPaymentMethod', $PaymentMethod );

    $Mockify->addMethodSpyWithParameterCheck('handleResponse', ['hashref']);

    return $Mockify->getMockObject();
}

sub _createShopTypeRef {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'UnityDemo'
    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::ShopTypeRef', $hAttributes
    );

    return $MockifySystem->getMockObject();
}


#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
