#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::PaymentStatusService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for UI::Provider
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::PaymentStatusService;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use Test::Mockify;
use Test::Mockify::Matcher qw ( HashRef Object String );
use Test::Mockify::Verify qw ( WasCalled GetParametersFromMockifyCall );
use Test::Exception;
use Test::MockObject::Extends;
use Test::More;
use DE_EPAGES::MolliePayments::API::Constants qw (
    LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS
    PAYMENT_METHOD_MOLLIE_PAYMENTS
    PAYMENT_STATUS_FAILED
    PAYMENT_STATUS_PENDING
    PAYMENT_STATUS_PAID
);

use strict;

use constant LINE_ITEM_PAYMENT_GUID => 'AJBD-IOHH-435-IUGU';
use constant MOLLIE_ID => 'tr_d0b0E3EA3v';
use constant ORDER_STATUS_OPEN => 'open';

sub test {
    my $self = shift;

    $self->test_handleStatusOnPaymentWebhook_errors();
    $self->test_handleStatusOnPaymentWebhook_success();

    $self->test_handleLineItemPaymentStatusChange_pending_paid();
    $self->test_handleLineItemPaymentStatusChange_pending_failed();
    $self->test_handleLineItemPaymentStatusChange_undef_failed();
    $self->test_handleLineItemPaymentStatusChange_undef_paid();
    $self->test_handleLineItemPaymentStatusChange_undef_pending();
    $self->test_handleLineItemPaymentStatusChange_empty_failed();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_handleStatusOnPaymentWebhook_errors {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
        'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };
    my $SUT = $self->_createSUT(my $WrongLineItemPayment = $self->_createOrder(), $hOrderResponse);
    throws_ok(
        sub { $SUT->handleStatusOnPaymentWebhook(LINE_ITEM_PAYMENT_GUID, MOLLIE_ID); },
        qr/Wrong LineItemPayment class/,
        $SubTestName . ' - Error gets thrown when Wrong LineItemPayment object'
    );

    $SUT = $self->_createSUT($self->_createLineItemPaymentMolliePayments(), $hOrderResponse);
    throws_ok(
        sub { $SUT->handleStatusOnPaymentWebhook(LINE_ITEM_PAYMENT_GUID, undef); },
        qr/WebhookPaymentId/,
        $SubTestName . ' - Error gets thrown when WebhookPaymentId is undef'
    );

    $SUT = $self->_createSUT($self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_FAILED), $hOrderResponse);
    throws_ok(
        sub { $SUT->handleStatusOnPaymentWebhook(LINE_ITEM_PAYMENT_GUID, MOLLIE_ID); },
        qr/already has a definitive status: failed/,
        $SubTestName . ' - Error gets thrown when LineItemPayment has status failed'
    );

    return;
}

sub test_handleStatusOnPaymentWebhook_success {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
        'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_PENDING);
    my $SUT = $self->_createSUT($LineItemPayment, $hOrderResponse);
    lives_ok(
        sub { $SUT->handleStatusOnPaymentWebhook(LINE_ITEM_PAYMENT_GUID, MOLLIE_ID);},
        "$SubTestName - test if handleStatusOnPaymentWebhook can be called without errors"
    );

    ok(WasCalled($SUT, '_getMollieOrderById'),
        "$SubTestName - _getMollieOrderById in handleStatusOnPaymentWebhook was called."
    );

    ok(WasCalled($SUT, 'handleLineItemPaymentStatusChange'),
        "$SubTestName - handleLineItemPaymentStatusChange in handleStatusOnPaymentWebhook was called."
    );

    is_deeply(  GetParametersFromMockifyCall( $SUT, 'handleLineItemPaymentStatusChange' ),
                [ $LineItemPayment, PAYMENT_STATUS_PAID, ORDER_STATUS_OPEN ],
                "$SubTestName - Methods withProfileId was called with correct ProfileId"
    );

    is($LineItemPayment->get('TransID'),
        MOLLIE_ID,
        "$SubTestName - LineItemPayment TransID was correctly updated."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_pending_paid {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_PENDING);
    my $SUT = $self->_createSUT($LineItemPayment, $hOrderResponse);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, PAYMENT_STATUS_PAID, ORDER_STATUS_OPEN);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_PAID,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_PAID,
        "$SubTestName - LineItemPayment was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    is(isValid($PaymentTransTime),
        1,
        "$SubTestName - LineItemPayment TransTime was correctly updated."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        $PaymentTransTime,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok((not WasCalled($SUT, '_cancelMollieOrderById')),
        "$SubTestName - _cancelMollieOrderById on paid status was not called."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_pending_failed {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_FAILED
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(PAYMENT_STATUS_PENDING);
    my $SUT = $self->_createSUT($LineItemPayment, $hOrderResponse);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, PAYMENT_STATUS_FAILED, ORDER_STATUS_OPEN);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_FAILED,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_FAILED,
        "$SubTestName - LineItemPayment TransStatus was correctly updated to failed");

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    is($PaymentTransTime,
        undef,
        "$SubTestName - LineItemPayment TransTime is undef."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn is undef."
    );

    ok(WasCalled($SUT, '_cancelMollieOrderById'),
        "$SubTestName - _cancelMollieOrderById on failed status was called."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_undef_paid {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PAID
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef);
    my $SUT = $self->_createSUT($LineItemPayment, $hOrderResponse);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, PAYMENT_STATUS_PAID, ORDER_STATUS_OPEN);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_PAID,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_PAID,
        "$SubTestName - LineItemPayment was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    is(isValid($PaymentTransTime),
        1,
        "$SubTestName - LineItemPayment TransTime was correctly updated."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        $PaymentTransTime,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(( not WasCalled($SUT, '_cancelMollieOrderById')),
        "$SubTestName - _cancelMollieOrderById on paid status was not called."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_empty_failed {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_FAILED
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments('');
    my $SUT = $self->_createSUT($LineItemPayment, $hOrderResponse);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, PAYMENT_STATUS_FAILED, ORDER_STATUS_OPEN);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_FAILED,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_FAILED,
        "$SubTestName - LineItemPayment TransStatus was correctly updated."
    );

    is($LineItemPayment->get('TransID'),
        undef,
        "$SubTestName - LineItemPayment TransID was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    is($PaymentTransTime,
        undef,
        "$SubTestName - LineItemPayment TransTime was correctly updated."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(WasCalled($SUT, '_cancelMollieOrderById'),
        "$SubTestName - _cancelMollieOrderById on failed status was called."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_undef_failed {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_FAILED
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef);
    my $SUT = $self->_createSUT($LineItemPayment, $hOrderResponse);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, PAYMENT_STATUS_FAILED, ORDER_STATUS_OPEN);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_FAILED,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_FAILED,
        "$SubTestName - LineItemPayment TransStatus was correctly updated to failed."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    is($PaymentTransTime,
        undef,
        "$SubTestName - LineItemPayment TransTime was correctly updated."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(WasCalled($SUT, '_cancelMollieOrderById'),
        "$SubTestName - _cancelMollieOrderById on failed status was called."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_undef_pending {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_PENDING
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef);
    my $SUT = $self->_createSUT($LineItemPayment, $hOrderResponse);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, PAYMENT_STATUS_PENDING, ORDER_STATUS_OPEN);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_PENDING,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_PENDING,
        "$SubTestName - LineItemPayment was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    is($PaymentTransTime,
        undef,
        "$SubTestName - LineItemPayment TransTime was correctly updated."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(( not WasCalled($SUT, '_cancelMollieOrderById')),
        "$SubTestName - _cancelMollieOrderById on failed status was not called."
    );

    return;
}

sub test_handleLineItemPaymentStatusChange_errors {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hOrderResponse = {
    'status' => ORDER_STATUS_OPEN,
        'payments' => [{
            'id' => MOLLIE_ID,
            'status' => PAYMENT_STATUS_FAILED
        }]
    };

    my $LineItemPayment = $self->_createLineItemPaymentMolliePayments(undef);
    my $SUT = $self->_createSUT($LineItemPayment, $hOrderResponse);
    my $StatusResult;
    lives_ok(
        sub { $StatusResult = $SUT->handleLineItemPaymentStatusChange($LineItemPayment, PAYMENT_STATUS_FAILED, ORDER_STATUS_OPEN);},
        "$SubTestName - test if handleLineItemPaymentStatusChange can be called without errors"
    );

    is($StatusResult,
        PAYMENT_STATUS_FAILED,
        "$SubTestName - $StatusResult was correctly returned."
    );

    is($LineItemPayment->get('TransStatus'),
        PAYMENT_STATUS_FAILED,
        "$SubTestName - LineItemPayment was correctly updated."
    );

    my $PaymentTransTime = $LineItemPayment->get('TransTime');
    is($PaymentTransTime,
        undef,
        "$SubTestName - LineItemPayment TransTime was correctly updated."
    );

    is($LineItemPayment->container()->parent()->get('PaidOn'),
        undef,
        "$SubTestName - Order PaidOn was correctly updated."
    );

    ok(WasCalled($SUT, '_cancelMollieOrderById'),
        "$SubTestName - _cancelMollieOrderById on failed status was called."
    );

    return;
}


#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;
    my ($LineItemPayment, $hOrderResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::PaymentStatusService');

    $Mockify->mock('_LogPayment')->when(String(), HashRef())->thenReturnUndef();

    $Mockify->mock('_LoadObjectByGUID')->when(String(LINE_ITEM_PAYMENT_GUID))->thenReturn($LineItemPayment);

    $Mockify->spy('_getMollieOrderById')
        ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'));

    $Mockify->spy('_cancelMollieOrderById')
        ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'));

    $Mockify->spy('handleLineItemPaymentStatusChange')
        ->when(Object('DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments'), String(), String());

    $Mockify->mock('_createMollieService')
        ->when(Object('DE_EPAGES::MolliePayments::API::Requests::Orders'))
        ->thenReturn($self->_createMolliePaymentsService($hOrderResponse));

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createMolliePaymentsService {
    my $self = shift;
    my ($hResponse) = @_;

    my $Mockify = Test::Mockify->new('DE_EPAGES::MolliePayments::API::MolliePaymentsService');

    $Mockify->mock('doGetRequest')->when()->thenReturn($hResponse);

    $Mockify->mock('doDeleteRequest')->when()->thenReturn($hResponse);

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createOrder {
    my $self = shift;

    my $hAttributes = {
        'instanceOf' => 'CustomerOrder',
        'PaidOn' => undef
    };
    my $MockifyOrder = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::CustomerOrder', $hAttributes
    );

    return $MockifyOrder->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my $self = shift;

    my $hAttributes = {};
    my $MockifyContainer = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes
    );
    $MockifyContainer->addMockWithReturnValue('parent', $self->_createOrder());

    return $MockifyContainer->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createLineItemPaymentMolliePayments {
    my $self = shift;
    my ($TransStatus) = @_;

    my $hAttributes = {
        'PaymentMethod' => $self->_createPaymentMethod(),
        'instanceOf' => LINE_ITEM_PAYMENT_MOLLIE_PAYMENTS,
        'TransStatus' => $TransStatus,
        'MollieOrderId' => MOLLIE_ID,
        'TransTime' => undef,
        'TransID' => undef,
        'LineItemContainer' => $self->_createLineItemContainer(),
        'IsTestTransaction' => 1,
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::LineItemPaymentMolliePayments', $hAttributes
    );

    return $Mockify->getMockObject();
}
#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'ID' => 123,
        'instanceOf' => PAYMENT_METHOD_MOLLIE_PAYMENTS
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

__PACKAGE__->runScript();
