#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::Requests_Profiles
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::MolliePayments::API::Requests::Profiles
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::Requests_Profiles;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::MockifyEpagesObject;

use Test::Mockify::Sut;
use Test::More;

use strict;

use constant HTTP_GET_METHOD => 'GET';
use constant ACCESS_TOKEN => 'MyAccessToken';

use constant REFRESH_TOKEN => 'MyRefreshtoken';

use constant PROFILE_ID => 'pfl_v9hTwCvYq1';

use constant PROFILES_SUFFIX_EPAGESJ_PATH => '/profiles';

use constant CLIENT_ID => 'MyClientID';

use constant CLIENT_SECRET => 'MyClientSecret';

use constant CLIENT_REDIRECT_URI =>
        'https://myshop.epages.de/epages/Site.admin/?ChangeAction=BacklinkMolliePaymentsOnboarding';

use constant RESPONSE_HASH => {
    'profiles' => [
        {
            'id' => 'pfl_v9hTwCvYq1',
            'name' => 'My website name',
            'website' => 'https://www.mywebsite.com',
        },
        {
            'id' => 'pfl_v9hTwCvYq2',
            'name' => 'My website name',
            'website' => 'https://www.mywebsite.com',
        }
    ],
        'next' => 'pfl_v9hTwCvYq3',
        'previous' => undef,
};

use constant HTTP_HEADERS_HASH => {
        'X-AccessToken' => ACCESS_TOKEN
};

sub test {
    my $self = shift;

    $self->test_Profiles_creation();
    $self->test_Profiles_provision_for_get_path_and_headers();
    $self->test_Profiles_provision_with_query_params_from_and_limit();
    $self->test_Profiles_provision_for_shop();
    $self->test_Profiles_handling_for_response();

    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_Profiles_creation {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    is( scalar(keys %{$SUT->{'data'}}), 1, "$SubTestName - Profiles object gets initialized with some arguments.");
    is( scalar(keys %{$SUT->{'queryParams'}}), 0, "$SubTestName - Profiles object gets initialized with an empty queryParams.");
    is( scalar(keys %{$SUT->{'payload'}}), 0, "$SubTestName - Profiles object gets initialized with an empty payload.");
}

sub test_Profiles_provision_for_get_path_and_headers {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    is(
            $SUT->getEpagesjRequestSuffixUrlPath('GET'),
            PROFILES_SUFFIX_EPAGESJ_PATH,
            "$SubTestName - Profiles object provides epagesj suffix path."
    );

    is_deeply( $SUT->getHttpHeaders('GET'), HTTP_HEADERS_HASH, "$SubTestName - Profiles object provides HTTP headers.");
}

sub test_Profiles_provision_with_query_params_from_and_limit {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $QueryParams = $SUT
            ->withFrom(PROFILE_ID)
            ->withLimit(1)
            ->getQueryParams(HTTP_GET_METHOD);


    is( $QueryParams->{'from'}, PROFILE_ID, "$SubTestName - Profiles query param from is retrieved.");

    is( $QueryParams->{'limit'}, 1, "$SubTestName - Profiles query param limit is retrieved.");
}

sub test_Profiles_provision_for_shop {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();
    my $Shop = $SUT->getShop();

    is( $Shop->alias(), 'MyShop', "$SubTestName - Profiles object provides shop object.");
}

sub test_Profiles_handling_for_response {
    my $self = shift;
    my $SubTestName = (caller(0))[3];

    my $SUT = $self->_createSUT();

    is_deeply(
            $SUT->handleResponse(RESPONSE_HASH),
            RESPONSE_HASH,
            "$SubTestName - Profiles response filtering provides original response."
    );
}


#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------
sub _createSUT {
    my $self = shift;

    my $hData = $self->_generateRequestData();
    my $aParameterList = [$hData];
    my $MockifySut = Test::Mockify::Sut->new('DE_EPAGES::MolliePayments::API::Requests::Profiles', $aParameterList);

    return $MockifySut->getMockObject();
}

sub _generateRequestData {
    my $self = shift;

    return {
        'payment' => $self->_createPaymentMethod()
    };
}

sub _createPaymentMethod {
    my $self = shift;

    my $hAttributes = {
        'AccessToken' => ACCESS_TOKEN,
        'RefreshToken' => REFRESH_TOKEN,
        'ProfileId' => PROFILE_ID,
        'Site' => $self->_createShop()
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

sub _createShop {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'MyShop'
    };
    my $MockifyPaymentMethod = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    return $MockifyPaymentMethod->getMockObject();
}

#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
