#========================================================================================
# package      DE_EPAGES::MolliePayments::t::unit::MolliePaymentsService
# base         DE_EPAGES::Core::API::Test
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for DE_EPAGES::MolliePayments::API::MolliePaymentsService
#========================================================================================
package DE_EPAGES::MolliePayments::t::unit::Requests_RequestWrapper;
use base DE_EPAGES::Core::API::Test;

use DE_EPAGES::Test::Mock::Mockify qw ( WasCalled );
use Test::Mockify::Matcher qw ( Object String HashRef );
use DE_EPAGES::Test::Mock::MockifyEpagesObject;
use DE_EPAGES::Test::Mock::MockifyServlet;
use DE_EPAGES::MolliePayments::API::Requests::Profiles;
use DE_EPAGES::MolliePayments::API::Requests::Tokens;
use DE_EPAGES::Test::Mock::EpagesObject;

use JSON::Any;
use Test::Exception;
use Test::Mockify::Sut;
use Test::MockObject::Extends;
use Test::More;

use strict;

use constant POST_METHOD => 'POST';
use constant GET_METHOD => 'GET';
use constant ACCESS_TOKEN => 'FAKE_ACCESS_TOKEN';
use constant EXPIRED_ACCESS_TOKEN => 'FAKE_EXPIRED_ACCESS_TOKEN';
use constant NEW_ACCESS_TOKEN => 'FAKE_NEW_ACCESS_TOKEN';
use constant REFRESH_TOKEN => 'FAKE_REFRESH_TOKEN';
use constant EPAGESJ_SUFFIX_URL_PATH => '/epjsuffix/path';
use constant DECODED_JSON_RESPONSE_AS_HASH => { 'resfield1' => 'resvalue1', 'resfield2' => 'resvalue2' };
use constant HEADER_WITH_EXPIRED_TOKEN => { 'AccessToken' => EXPIRED_ACCESS_TOKEN };
use constant REQUEST_FAILED_CODE => 'REQUEST_FAILED';
use constant MOLLIE_ERROR_STRING => "MollieErrorResponse [status=401 | title=Unauthorized Request | detail=Missing authentication, or failed to authenticate | field=null]";
use constant HTTP_STATUS_CODE_424 => 424;

use constant EXPECTED_ERROR_RESULT_HASH => {
    'exception' => 'MollieErrorResponse',
    'status'    => HTTP_STATUS_CODE_424,
    'content'   => {
        'status' => '401',
        'title'  => 'Unauthorized Request',
        'detail' => 'Missing authentication, or failed to authenticate',
        'field'  => 'null'
    }
};

use constant TOKENS_RESPONSE_HASH => {
    'access_token' => NEW_ACCESS_TOKEN,
    'refresh_token' => REFRESH_TOKEN,
    'expires_in' => 3600,
    'token_type' => 'bearer',
    'scope' => 'payments.read payments.write refunds.read refunds.write profiles.read profiles.write'
};
sub test {
    my $self = shift;

    $self->test_DoPost_WithTokensRequest();
    $self->test_DoGet_WithProfilesRequest_ExpiredToken();
    done_testing();
}

#----------------------------------------------------------------------------------------
# Tests
#----------------------------------------------------------------------------------------
sub test_DoPost_WithTokensRequest {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    note("-------- doPost --------");

    my $MockedPaymentMethod = $self->_createPaymentMethod(
        {
            'RefreshToken' => REFRESH_TOKEN,
            'AccessToken' => ACCESS_TOKEN,
        },
        $self->_createShop([])
    );

    my $MockedTokensRequest = $self->_createMockedTokenRequest($MockedPaymentMethod);
    my $SUT = $self->_createSUT($MockedTokensRequest, $MockedTokensRequest);
    my $IsInTestMode = 0;
    my $Response = $SUT->doPost($IsInTestMode);

    ok(WasCalled($MockedTokensRequest, 'getEpagesjRequestSuffixUrlPath'),
        "$SubTestName - getEpagesjRequestSuffixUrlPath performed in doPost by the wrapper."
    );

    ok(WasCalled($MockedTokensRequest,'getHttpHeaders'),
        "$SubTestName - getHttpHeaders performed in doPost by the wrapper.");

    ok(WasCalled(
        $MockedTokensRequest, 'getPayload'),
        "$SubTestName - getPayload in RequestWrapper was called."
    );
    ok(WasCalled($SUT, '_doRequest'),
    "$SubTestName - _doRequest performed in doPost by the wrapper.");

    is_deeply($Response, TOKENS_RESPONSE_HASH, "$SubTestName - Wrapper response contains new access code.");

}

sub test_DoGet_WithProfilesRequest_ExpiredToken {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    note("-------- doGet --------");

    my $aPaymentMethodsToUpdate = [
        $self->_createDummyPaymentMethod(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments',
            {
                'RefreshToken' => REFRESH_TOKEN,
                'AccessToken'  => EXPIRED_ACCESS_TOKEN,
                'Alias'        => 'PaymentMethodMolliePayments',
                'instanceOf'   => 'PaymentMethodMolliePayments',
            }
        ),
        $self->_createDummyPaymentMethod(
            'DE_EPAGES::Mollie::API::Object::PaymentMethodMollie',
            {
                'Alias'        => 'PaymentMethodMollie',
                'AccessToken'  => EXPIRED_ACCESS_TOKEN,
                'RefreshToken' => REFRESH_TOKEN,
                'instanceOf'   => 'PaymentMethodMollie',
            }
        ),
        $self->_createDummyPaymentMethod(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments',
            {
                'RefreshToken' => 'Not to update eheh',
                'AccessToken'  => EXPIRED_ACCESS_TOKEN,
                'Alias'        => 'PaymentMethodMolliePayments',
                'instanceOf'   => 'PaymentMethodMolliePayments',
            }
        ),
        $self->_createDummyPaymentMethod(
            'DE_EPAGES::AmazonPayments::API::Object::PaymentMethodAmazonPayments',
            {
                'Alias'       => 'PaymentMethodAmazonPayments',
                'AccessToken' => EXPIRED_ACCESS_TOKEN,
                'instanceOf'  => 'PaymentMethodAmazonPayments',
            }
        ),
        $self->_createDummyPaymentMethod(
            'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments',
            {
                'RefreshToken' => REFRESH_TOKEN,
                'AccessToken'  => EXPIRED_ACCESS_TOKEN,
                'Alias'        => 'PaymentMethodMolliePayments',
                'instanceOf'   => 'PaymentMethodMolliePayments',
            }
        )
    ];

    my $MockedShop = $self->_createShop($aPaymentMethodsToUpdate);
    my $MockedPaymentMethod = $self->_createPaymentMethod(
        {
            'AccessToken' => EXPIRED_ACCESS_TOKEN,
            'RefreshToken' => REFRESH_TOKEN,
            'alias'      => 'PaymentMethodMolliePayments',
            'instanceOf' => 'PaymentMethodMolliePayments',
        },
        $MockedShop
    );

    my $ProfilesRequest = $self->_createProfilesRequest($MockedPaymentMethod);
    my $MockedTokensRequest = $self->_createMockedTokenRequest($MockedPaymentMethod, $MockedShop);

    my $SUT = $self->_createSUT($ProfilesRequest, $MockedTokensRequest);
    my $IsInTestMode = 1;
    my $Response;

    is($MockedPaymentMethod->get('AccessToken'), EXPIRED_ACCESS_TOKEN, "$SubTestName - Payment method EXPIRED AccessToken was correctly set.");
    is($aPaymentMethodsToUpdate->[0]->get('AccessToken'), EXPIRED_ACCESS_TOKEN, "$SubTestName - Payment method to Update [0] has an EXPIRED AccessToken correctly set.");

    lives_ok( sub {
            $Response = $SUT->doGet($IsInTestMode);
        },
        "$SubTestName - doGet called with Expired token lives ok."
    );

    ok(WasCalled($SUT, '_doRequest'),
        "$SubTestName - _doRequest called by doGet in the wrapper.");

    ok(WasCalled($SUT, '_GetError'),
        "$SubTestName - _GetError called by doGet in the wrapper.");

    ok(WasCalled($SUT, '_LogError'),
        "$SubTestName - _LogError called by doGet in the wrapper.");

    ok(WasCalled($SUT, '_isTokenExpired'),
        "$SubTestName - _isTokenExpired called by doGet in the wrapper.");

    ok(WasCalled($SUT, '_refreshPaymentAccessToken'),
        "$SubTestName - _refreshPaymentAccessToken called by doGet in the wrapper.");

    ok(WasCalled($SUT, '_createTokenRequest'),
        "$SubTestName - _createTokenRequest called by doGet in the wrapper.");

    ok(WasCalled($SUT, '_updatePaymentMethodsAccessToken'),
        "$SubTestName - _updatePaymentMethodsAccessToken called by doGet in the wrapper.");

    ok(WasCalled($MockedShop, 'folder'),
    "$SubTestName - folder called by _updatePaymentMethodsAccessToken on the Shop.");

    is($MockedPaymentMethod->get('AccessToken'), NEW_ACCESS_TOKEN, "$SubTestName - Payment method NEW AccessToken was correctly set.");
    is($aPaymentMethodsToUpdate->[0]->get('AccessToken'), NEW_ACCESS_TOKEN, "$SubTestName - Payment method to Update [0] has a NEW AccessToken correctly set.");
    is($aPaymentMethodsToUpdate->[1]->get('AccessToken'), EXPIRED_ACCESS_TOKEN, "$SubTestName - Payment method to Update [1] has a EXPIRED AccessToken not updated.");
    is($aPaymentMethodsToUpdate->[2]->get('AccessToken'), EXPIRED_ACCESS_TOKEN, "$SubTestName - Payment method to Update [2] has a EXPIRED AccessToken not updated.");
    is($aPaymentMethodsToUpdate->[3]->get('AccessToken'), EXPIRED_ACCESS_TOKEN, "$SubTestName - Payment method to Update [3] has a EXPIRED AccessToken not updated.");
    is($aPaymentMethodsToUpdate->[4]->get('AccessToken'), NEW_ACCESS_TOKEN, "$SubTestName - Payment method to Update [4] has a NEW AccessToken correctly set.");

}

#----------------------------------------------------------------------------------------
# Helpers
#----------------------------------------------------------------------------------------

sub _createPaymentMethod {
    my $self = shift;
    my ($hParams, $MockedShop) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
    'DE_EPAGES::MolliePayments::API::Object::PaymentMethodMolliePayments', $hParams
    );
    $Mockify->addMockWithReturnValue('getSite', $MockedShop);

    return $Mockify->getMockObject();
}

sub _createDummyPaymentMethod {
    my $self = shift;
    my ($Package, $hParam) = @_;

    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        $Package, $hParam
    );

    return $Mockify->getMockObject();
}

sub _createShop {
    my $self = shift;
    my ($aPaymentMethods) = @_;

    my $hAttributes = {
        'Alias' => 'MyShop',
        'ShopTypeRef' => $self->_createShopTypeRef()
    };
    my $Mockify = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
        'DE_EPAGES::Shop::API::Object::Shop', $hAttributes
    );

    my $Folder = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Object', {});
    $Folder->mock('children', sub { return $aPaymentMethods });

    $Mockify->addMockWithReturnValueAndParameterCheck('folder', $Folder, [{'string'=>'PaymentMethods'}]);
    return $Mockify->getMockObject();
}

sub _createSUT {
    my $self = shift;
    my ($RequestMock, $MockedTokensRequest) = @_;

    my $MockifySut = Test::Mockify::Sut->new(
        'DE_EPAGES::MolliePayments::API::Requests::RequestWrapper', [ $RequestMock ]
    );

    $MockifySut->mockImported('DE_EPAGES::Object::API::Factory', 'LoadRootObject')
        ->whenAny()
        ->thenReturn($self->_createSystem());

    $MockifySut->mockStatic('DE_EPAGES::EPagesJ::API::Object::RestClient::create')
        ->whenAny()
        ->thenReturn($self->_createRestClient());

    my $Error = DE_EPAGES::Core::API::Error->new(
        Code => REQUEST_FAILED_CODE,
        Vars => {
            'Status' => 424,
            'Response' => MOLLIE_ERROR_STRING
        }
    );

    $MockifySut->spy('_isTokenExpired')->when(Object());

    $MockifySut->spy('_refreshPaymentAccessToken')->when();

    $MockifySut->mock('_createTokenRequest')
            ->when(HashRef())
            ->thenReturn($MockedTokensRequest);

    $MockifySut->spy('_updatePaymentMethodsAccessToken')->when();

    $MockifySut->mock('_GetError')->whenAny()->thenReturn($Error);

    $MockifySut->mock('_ParseErrorStringToHash')->whenAny()->thenReturn(
        EXPECTED_ERROR_RESULT_HASH
    );

    $MockifySut->mock('_LogError')->whenAny()->thenReturnUndef();
    $MockifySut->mock('_LogPayment')->whenAny()->thenReturnUndef();
    # get profiles request with expired token should fail
    $MockifySut->mock('_doRequest')
        ->when(
            String(GET_METHOD),
            String('payments/mollie/v2/profiles'),
            HashRef({'X-AccessToken' => EXPIRED_ACCESS_TOKEN }),
            HashRef())
        ->thenThrowError(REQUEST_FAILED_CODE);
    # get profiles request with new token should return profiles dummy response
    $MockifySut->mock('_doRequest')
        ->when(
        String(GET_METHOD),
        String('payments/mollie/v2/profiles'),
        HashRef({'X-AccessToken' => NEW_ACCESS_TOKEN }),
        HashRef())
        ->thenReturn(DECODED_JSON_RESPONSE_AS_HASH);
    # post tokens request return tokens dummy response
    $MockifySut->mock('_doRequest')
        ->when(
            String(POST_METHOD),
            String('payments/mollie/v2/tokens'),
            HashRef({'Authorization' => 'Basic f3adfadsraradgfgh'}),
            HashRef())
        ->thenReturn(TOKENS_RESPONSE_HASH);

    return $MockifySut->getMockObject();
}

sub _createMockedTokenRequest {
    my $self = shift;
    my ($PaymentMethod, $MockedShop) = @_;

    my $aParameters = [{
        'payment' => $PaymentMethod,
        'code' => 'MyAuthorizationCode'
    }];
    my $Mockify = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::MolliePayments::API::Requests::Tokens', $aParameters);

    $Mockify->addMockWithReturnValue('getShop', $MockedShop);
    $Mockify->addMethodSpyWithParameterCheck('getEpagesjRequestSuffixUrlPath', ['string']);
    $Mockify->addMockWithReturnValueAndParameterCheck(
        'getHttpHeaders', {'Authorization' => 'Basic f3adfadsraradgfgh'} , ['string']
    );

    $Mockify->addMockWithReturnValueAndParameterCheck(
                'getPayload',
                {} ,
                ['string']
    );
    $Mockify->addMockWithReturnValue('getPaymentMethod', $PaymentMethod );
    return $Mockify->getMockObject();

}

sub _createProfilesRequest {
    my $self = shift;
    my ($PaymentMethod) = @_;

    my $hParameters = {
        'payment' => $PaymentMethod,
    };

    return DE_EPAGES::MolliePayments::API::Requests::Profiles->new($hParameters);
}

sub _createSystem {
    my $self = shift;

    my $hAttributes = {
        'EPagesJBaseURL' => 'https://shop.example.com:8088/rs',
        'OAuthServiceBaseUrl' => undef,
        'ProviderID' => 'epages',
        'MolliePayments_AppClientID' => 'CLIENT_ID',
        'MolliePayments_AppClientSecret' => 'CLIENT_SECRET',
        'MolliePayments_AppRedirectURL' => 'CLIENT_REDIRECT_URI'

    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Object::API::Object::Site', $hAttributes
    );

    return $MockifySystem->getMockObject();
}

sub _createRestClient {
    my $self = shift;

    my $aParameters = [];
    my $MockifyRestClient = DE_EPAGES::Test::Mock::Mockify->new(
            'DE_EPAGES::EPagesJ::API::Object::RestClient', $aParameters
    );
    $MockifyRestClient->addMockWithReturnValue('client', $self->_createHttpClient());

    return $MockifyRestClient->getMockObject();
}

sub _createHttpClient {
    my $self = shift;

    my $MockifyHttpClient = DE_EPAGES::Test::Mock::Mockify->new('DE_EPAGES::EPagesJ::API::Object::HTTPClient');

    $MockifyHttpClient->addMockWithReturnValueAndParameterCheck(
            'get', $self->_createHttpResponse, ['object', 'hashref']
    );
    $MockifyHttpClient->addMockWithReturnValueAndParameterCheck(
            'post', $self->_createHttpResponse, ['string', 'hashref', 'any']
    );
    $MockifyHttpClient->addMockWithReturnValueAndParameterCheck(
            'put', $self->_createHttpResponse, ['string', 'hashref', 'any']
    );

    return $MockifyHttpClient->getMockObject();
}

sub _createHttpResponse {
    my $self = shift;

    my $Json = JSON::Any->new();

    my $MockifyHttpResponse = DE_EPAGES::Test::Mock::Mockify->new('HTTP::Response');
    $MockifyHttpResponse->addMockWithReturnValue('decoded_content', $Json->objToJson(DECODED_JSON_RESPONSE_AS_HASH));

    return $MockifyHttpResponse->getMockObject();
}

sub _createShopTypeRef {
    my $self = shift;

    my $hAttributes = {
        'Alias' => 'UnityDemo'
    };
    my $MockifySystem = DE_EPAGES::Test::Mock::MockifyEpagesObject->new(
            'DE_EPAGES::Shop::API::Object::ShopTypeRef', $hAttributes
    );

    return $MockifySystem->getMockObject();
}


#========================================================================================
# Main
#========================================================================================

__PACKAGE__->runScript();
