#========================================================================================
# package      DE_EPAGES::Object::API::XML::ObjectImportHandler
# state        public
# base         DE_EPAGES::XML::API::BaseImportHandler
#----------------------------------------------------------------------------------------
# description  This module implements a SAX2 handler to import object type(s)
#               Object, Class, Attribute.
#
# example      use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
#               ImportObjects( 'Objects.xml' );
#========================================================================================
package DE_EPAGES::Object::API::XML::ObjectImportHandler;
use base qw ( DE_EPAGES::XML::API::BaseImportHandler Exporter );
use strict;

our @EXPORT_OK = qw (
    ImportObjects
);
use JSON;
use XML::SAX::ParserFactory;
use DE_EPAGES::Cartridge::API::XML::CartridgesImportHandler;
use DE_EPAGES::Core::API::PerlTools qw( ElementOf unique );
use DE_EPAGES::Core::API::Error qw( VarsOnError Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw( GetLog );
use DE_EPAGES::Core::API::TestParameter qw( TestString );
use DE_EPAGES::Object::API::Language qw( GetPKeyLanguageByCode );
use DE_EPAGES::Object::API::Factory qw (LoadObject LoadObjectByPath LoadClassByAlias Transaction ExistsClassByAlias);
use DE_EPAGES::XML::API::DateTimeFormatXML;
use DE_EPAGES::Object::API::XML::StandardImportHandler;##no critic (RequireCorrectImports)
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Trigger::API::BatchProcessor qw (RunBatchProcess);

my $TIME_PARSER = DE_EPAGES::XML::API::DateTimeFormatXML->new;

sub ErrorDefinition {
    return {
        'FileNotFound' => 'File #FileName doesn\'t exist.',
        'REF_OBJECT_PATH_NOT_FOUND' => 'Referenced object path not found #Path at object #Alias (Parent=#Parent,AttributeName=#AttributeName)',
    }
}

#========================================================================================
# function     ImportObjects
# state        public
#----------------------------------------------------------------------------------------
# syntax       ImportObjects( $FileName );
# syntax       ImportObjects( $FileName, $Path );
# syntax       ImportObjects( $FileName, $Path, $cTransform );
# example      ImportObjects( 'DemoShop.xml', '/Shops/DemoShop' );
#----------------------------------------------------------------------------------------
# description  Imports objects from an XML file.
#----------------------------------------------------------------------------------------
# input        $FileName | XML file name | string
# input        $Path | (optional) path to parent object | string
# input        $cTransform | (optional) transformer function, works like a hook for xml-imports making it possible
#                              to react to the occurence of certain tags of attribute values - params are: <ul>
#                           <li>Object - object - DE_EPAGES::Object::API::Object::Object</li>
#                           <li>LocalName - tag name - String</li>
#                           <li>Attribute - this tags attributes - ref.hash</li>
#                   | ref.code
#========================================================================================
sub ImportObjects ($;$$) {
    my ($FileName, $Path, $cTransform ) = @_;
    my $Handler = __PACKAGE__->new;
    $Handler->importXML($FileName, $Path, $cTransform);
}

#========================================================================================
# function     importXML
# state        private
#----------------------------------------------------------------------------------------
# syntax       $class->importXML( $FileName );
# syntax       $class->importXML( $FileName, $Path );
# example      DE_EPAGES::Object::API::XML::ObjectImportHandler->importXML( 'Products.xml', '/Shops/DemoShop' );
#----------------------------------------------------------------------------------------
# description  Imports objects from an XML file.
#----------------------------------------------------------------------------------------
# input        $FileName | XML file name | string
# input        $Path | (optional) path to parent object | string
# input        $cTransform | (optional) transformer function | ref.code
#========================================================================================
sub importXML {
    my $class = shift;
    my ($FileName, $Path, $cTransform) = @_;
    TestString( 'FileName', $FileName );
    Error( 'FileNotFound', { FileName => $FileName } ) unless -f $FileName;
    GetLog->debug(sprintf 'importXML with Package "%s", File "%s", Path "%s".', $class, $FileName, $Path // 'undef');

    $Path = '/' unless defined $Path;
    my $Object = LoadObjectByPath($Path);

    my $Handler = $class->new;
    $Handler->pushID('Object', $Object);
    $Handler->pushID('ObjectID', $Object->id);
    $Handler->{'FileName'} = $FileName;
    $Handler->{'TransForm'} = $cTransform;

    TriggerHook('API_BeforeImportXML', { Handler => $Handler });

    my $Parser = XML::SAX::ParserFactory->parser( Handler => $Handler );
    eval {
        RunBatchProcess('ImportXML', sub {
            GetLog()->debug('batch importing starts..');
            $Parser->parse_uri( $FileName );
        });
    };
    VarsOnError($Handler->varsOnError()) if ExistsError();
    return;
}

sub start_element {
    my $self = shift;
    my ($el) = @_;

    if ($self->topID( 'DiedAtOptionalTag')) {
        $self->pushID( 'DiedAtOptionalTag', 1);
        return;
    }

    my $hAttributes = $self->getAttributes( $el );
    my $IsOptional  = defined $hAttributes->{'optional'}  ? $hAttributes->{'optional'}  : 0;
    $self->pushID( 'IsOptional',  $IsOptional);

    if (defined $self->{'TransForm'}) {
        $self->{'TransForm'}->($self->topID('Object'), $el->{'LocalName'}, $hAttributes);
    }

    my $DiedAtOptionalTag = 0;
    eval { $self->SUPER::start_element($el); };
    if (ExistsError()) {
        my $Error = GetError();
        if ($IsOptional && $Error->code eq 'NoResults_Attribute') {
            # EPG-34806: Allow invalid attribute names in the tag with optional=1,
            # but not in sub elements.
            GetLog->debug($Error->shortMessage);
        }
        elsif ($IsOptional && $Error->code ~~ [qw( FeatureLimitExceeded PATH_NOT_FOUND )]) {
            # PATH_NOT_FOUND can happen as a follow-up error when a shiping method (or anything else)
            # was not created (because of FeatureLimitExceeded), but is referenced later in the XML file
            GetLog->debug($Error->shortMessage);
            $DiedAtOptionalTag = 1;
            $self->{'SkipXMLStackCheck'} = 1; # EPG-30858
        }
        else {
            Error($Error);
        }
    }

    $self->pushID( 'DiedAtOptionalTag',  $DiedAtOptionalTag);
}

sub end_element {
    my $self = shift;
    my ($el) = @_;

    my $Died = $self->popID( 'DiedAtOptionalTag' );

    $self->SUPER::end_element($el) unless($Died);

    $self->popID( 'IsOptional' );
}

sub startUnknownTag {
    my $self = shift;
    my ($el) = @_;

    $self->start_Object($el);
}

sub endUnknownTag {
    my $self = shift;
    my ($el) = @_;

    $self->end_Object($el);
}

#========================================================================================
# Cartridge
#========================================================================================
sub start_Cartridge { ## no critic qw(RequireArgUnpacking)
    return DE_EPAGES::Cartridge::API::XML::CartridgesImportHandler::start_Cartridge( @_ );
}

sub end_Cartridge { ## no critic qw(RequireArgUnpacking)
    return DE_EPAGES::Cartridge::API::XML::CartridgesImportHandler::end_Cartridge( @_ );
}

#========================================================================================
# Object
#========================================================================================
sub start_Object {
    my $self = shift;
    my ($el) = @_;
    my $ClassTag = $el->{'LocalName'};

    my $hAttributes = $self->getAttributes( $el );
    my $IsReference = defined $hAttributes->{'reference'} ? $hAttributes->{'reference'} : 0;
    my $ClassName   = defined $hAttributes->{'Class'} ? $hAttributes->{'Class'} : $ClassTag;

    die "Attributes 'reference' and 'delete' are set both on element $ClassTag"
        if $IsReference && $hAttributes->{'delete'};

    my $DoImport=1;
    my $Class;

    if ( $ClassName =~ /\// ) {
        $Class = $self->topID( 'Object' )->path( $ClassName );
    } else {
        if ($hAttributes->{'optional'} == 1 and not ExistsClassByAlias($ClassName)) {
            $DoImport=0;
        } else {
          $Class=LoadClassByAlias( $ClassName );
        }
    }
    delete $hAttributes->{'optional'};
    Error('NO_CLASS', $hAttributes) if ($DoImport and not $Class->instanceOf( 'Class' ));

    $self->pushID( 'IsReference', $IsReference);

    my $Object;
    Transaction( sub {
        # copy the original attributes, because some implementations of
        # the importXML() method (e.g. Product) clobber the input hash ref.
        # in case of a deadlock-retry the attributes would be lost
        my %Attributes = %$hAttributes;
        $Object = $Class->packageObject->importXML($self, $el, $Class, \%Attributes);
    }) if $DoImport;

    if (defined $Object) {
        $self->pushID( 'Object',  $Object);
        $self->pushID( 'ObjectID',$Object->id);
    } else {
        $self->pushID( 'Object', $self->topID( 'Object'));
        $self->pushID( 'ObjectID',$self->topID( 'ObjectID'));
    }

    return $Object;
}

sub end_Object {
    my $self = shift;
    my ($el) = @_;

    $self->topID('Object')->importXMLEnd($self, $el);

    $self->popID( 'IsReference' );
    $self->popID( 'Object' );
    $self->popID( 'ObjectID');
}

#========================================================================================
# function     isReference
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsReference = $Package->isReference;
#----------------------------------------------------------------------------------------
# description  Is current element imported as reference ?
#----------------------------------------------------------------------------------------
# return       $IsReference | true if the current element is marked as reference | boolean
#========================================================================================
sub isReference {
    my $self = shift;
    return $self->topID( 'IsReference' );
}

#========================================================================================
# function     isOptional
# state        public
#----------------------------------------------------------------------------------------
# syntax       $IsOptional = $Package->isOptional;
#----------------------------------------------------------------------------------------
# description  Is current element optional imported ?
#----------------------------------------------------------------------------------------
# return       $IsOptional | true if the current element is marked as optional | boolean
#========================================================================================
sub isOptional {
    my $self = shift;
    return $self->topID( 'IsOptional' );
}

#========================================================================================
# AttributeValue
#========================================================================================
sub start_AttributeValue {
    my $self = shift;
    my ($hElement) = @_;

    my $hAttributes = $self->getAttributes( $hElement );
    my $Object = $self->topID('Object');

    my $LanguageID;
    if( exists $hAttributes->{'Language'} ) {
        $LanguageID = GetPKeyLanguageByCode( lc $hAttributes->{'Language'} );
    }

    # get the class from the Class attribute
    die 'Missing attribute Name on element Attribute' unless exists $hAttributes->{'Name'};

    if( exists $hAttributes->{'Value'} ) {
        $hAttributes->{'Value'} = undef unless $hAttributes->{'Value'} =~ /\S/;
        $self->_setAttributeValues( $Object, { $hAttributes->{'Name'} => $hAttributes->{'Value'} }, $LanguageID );
    }
    else {
        $self->collectCharacters( 1 );
        $self->pushID( 'AttributeName', $hAttributes->{'Name'} );
        $self->pushID( 'LanguageID', $LanguageID );
    }

    return;
}

sub end_AttributeValue {
    my $self = shift;

    if( $self->collectCharacters ) {
        my $Value = $self->getCharacters;
        $self->collectCharacters( 0 );

        my $Object = $self->topID( 'Object' );
        my $Name = $self->popID( 'AttributeName' );
        my $LanguageID = $self->popID( 'LanguageID' );

        my $hAttributes = { $Name => $Value };
        $self->_setAttributeValues( $Object, $hAttributes, $LanguageID );
    }
}

#========================================================================================
# AttributeLanguage
#========================================================================================
sub start_AttributeLanguage {
    my $self = shift;
    my ($hElement) = @_;

    my $hAttributes = $self->getAttributes( $hElement );
    my $Object = $self->topID('Object');
    my $LanguageID;
    if( exists $hAttributes->{'Language'} ) {
        $LanguageID = GetPKeyLanguageByCode( lc $hAttributes->{'Language'} );
        delete $hAttributes->{'Language'};
    }

    $self->_setAttributeValues($Object, $hAttributes, $LanguageID);
    return;
}

sub end_AttributeLanguage {}

#========================================================================================
# AttributeLoop
#========================================================================================
sub start_AttributeLoop {
    my $self = shift;
    my ($hElement) = @_;

    my $hAttributes = $self->getAttributes( $hElement );
    $self->pushID( 'Attribute', $hAttributes);

    return;
}

sub end_AttributeLoop {
    my $self = shift;

    my $aItems = $self->stackReturn('AttributeLoopItem');
    my $Attribute = $self->topID('Attribute');
    my $Name = $Attribute->{'Name'};
    my $Object = $self->topID('Object');
    if ($Attribute->{'increment'}) {
        my $aElements = $Object->get($Name);
        unshift @$aItems, @$aElements;
        if ($Attribute->{'unique'}) {
            @$aItems = unique @$aItems;
        }
    }
    $Object->set({ $Name => $aItems });
    return;
}

#========================================================================================
# AttributeLoopItem
#========================================================================================
sub start_AttributeLoopItem {
    my $self = shift;
    my ($hElement) = @_;

    my $hAttributes = $self->getAttributes( $hElement );
    my $Object = $self->topID('Object');
    my $Item = LoadObject($Object->findPath($hAttributes->{'Path'}));
    $self->pushReturn('AttributeLoopItem', $Item);
    return;
}

sub end_AttributeLoopItem {}

#========================================================================================
# function     _setAttributeValues
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->_setAttributeValues($Object, $hAttributes, $LanguageID);
#----------------------------------------------------------------------------------------
# description  Converts date/time data types,
#               Calls $Object->class->importXMLPackages() before $Objetc->set().
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $hAttributes | attribute values | ref.hash
# input        $LanguageID | (optional) lanuage id | int
#========================================================================================
sub _setAttributeValues {
    my $self = shift;
    my ($Object, $hAttributes, $LanguageID) = @_;

    my $Class = $Object->class;
    $Class->importXMLPackages($self, $hAttributes);

    foreach my $Key (keys %$hAttributes) {
        my $Attribute = $Class->attribute($Key);
        if ($Attribute->isObject && !$Attribute->isArray) {
            if (defined $hAttributes->{$Key}) {
                my $refObjectID = $Object->findPath($hAttributes->{$Key});
                if (defined $refObjectID) {
                    $hAttributes->{$Key} = LoadObject($refObjectID);
                }
                else {
                    Error('REF_OBJECT_PATH_NOT_FOUND', {
                        'Path'  => $hAttributes->{$Key},
                        'Alias' => $Object->alias,
                        'Parent' => $Object->pathString,
                        'AttributeName' => $Key,
                    });
                }
            }
        }
        elsif (ElementOf($Attribute->type, 'DateTime', 'Date', 'Time', 'Duration')) {
            $hAttributes->{$Key} = $hAttributes->{$Key} ne ''
                ? $TIME_PARSER->parse_datetime($hAttributes->{$Key})
                : undef;
        }
        elsif ($Attribute->type eq 'JSON') {
            $hAttributes->{$Key} = from_json($hAttributes->{$Key});
        }
    }

    $Object->set( $hAttributes, $LanguageID );
    return;
}

1;
