#==============================================================================
# package      DE_EPAGES::Order::UI::OrderProcess
# state        public
# description  UI class for Basket Order Process
#==============================================================================
package DE_EPAGES::Order::UI::OrderProcess;
use base DE_EPAGES::Order::UI::Basket_Ext;

use strict;
use DE_EPAGES::Core::API::Error qw( Error );
use DE_EPAGES::Core::API::PerlTools qw ( Compare );
use DE_EPAGES::Object::API::Factory qw( LoadClassByAlias );
use DE_EPAGES::Order::API::LineItemContainer qw( CheckSavePayment CheckSaveShipping UpdateTaxModel );
use DE_EPAGES::Order::API::Object::BasketOrderProcess;
use DE_EPAGES::Order::UI::Basket;
use DE_EPAGES::Presentation::API::Session qw( SetSessionAttribute );
use DE_EPAGES::Shop::API::Constants qw( TAX_MODEL_NET );
use DE_EPAGES::Trigger::API::Trigger qw( TriggerHook );

#==============================================================================
# ChangeActions for order process
#==============================================================================

sub SaveMultiCheckoutBasket {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook for additional test during this action
    TriggerHook('UI_SaveMultiCheckoutBasket', {'Servlet' => $Servlet});

    # save Basket (update)
    my $Basket = $Servlet->object;
    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'ViewObjectID' => $Basket->id,
            'OrderGUID' => $Basket->get('OrderGUID'),
        },
        'ViewAction' => 'ViewMultiCheckoutBlocked',      #new OrderConfirmationPage???
    }) if defined $Basket->get('OrderGUID');

    $self->SUPER::Save($Servlet);


    # trigger hook for all basket tests wich are done twice
    TriggerHook('UI_AfterSaveMultiCheckoutBasket', {'Servlet' => $Servlet});

    # do order, if the DoOrderStep value is set to the current process
    $self->CreateOrder($Servlet, 'Basket');

    return;
}

sub SaveMultiCheckoutAddress {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;

    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'ViewObjectID' => $Basket->id,
            'OrderGUID' => $Basket->get('OrderGUID'),
        },
        'ViewAction' => 'ViewBlocked',
    }) if defined $Basket->get('OrderGUID');

    # newsletter is subscribed in checkout
    $Servlet->params('SubscribedInCheckout', 1);
    my $User = $Servlet->user;

    # trigger hook for additional test during this action
    TriggerHook('UI_SaveMultiCheckoutAddress', {'Servlet' => $Servlet});

    # control selection of login type (normal login, address login with or without registration)
    my $CustomerType = $Servlet->params('CustomerType');

    # do not execute this action if we only login the customer (done by ChangeAction SaveLoginForm )
    return if defined $CustomerType and $CustomerType eq 'Member';

    if (!defined $CustomerType && $User->get('IsAnonymous')) {
        $Servlet->form->executeFormError({
            'Name'       => 'CustomerType',
            'Reason'     => 'NoCustomerTypeSelection',
            'ViewAction' => 'ViewMultiCheckoutAddress'
        });
    }
    else {
        # when comming back to viewaction the right type should be selected
        SetSessionAttribute( $Servlet->intSessionID, 'CustomerType', $CustomerType );
    }

    # add save and test functions here

    my $CurrentShippingAddress = $Basket->get('ShippingAddress');
    #saves address selection only for existing addresses
    if ($Servlet->params('ShippingAddress') and $Servlet->params('BillingAddress')) {
        $Servlet->doAction('SelectAddress');
    }

    # reset the shipping address to the new address, because SelectAddress() overwrites the setting
    if ($Servlet->params('EnableShippingAddress')){
        $Basket->set({'ShippingAddress' => $CurrentShippingAddress});
    }

    # correct shipping address on user
    # for unity we update also the shipping address on the user
    #(because there is no difference between addresses for user/customer/basket/order)
    if ($Basket->getSite()->get('IsUnityRegistered')){
        my $CurrentBasketShippingAddress = $Basket->get('ShippingAddress');
        my $CurrentBasketBillingAddress = $Basket->get('BillingAddress');

        # we do not save same addresses on the user
        if (defined $CurrentBasketBillingAddress and defined $CurrentBasketShippingAddress
            and $CurrentBasketBillingAddress->id eq $CurrentBasketShippingAddress->id){
            $CurrentBasketShippingAddress =  undef;
        }

        # set shipping address on user if we have an extra shipping address on the basket
        $User->set({'ShippingAddress' => $CurrentBasketShippingAddress});
    }

    # test address
    my $Shop = $Basket->getSite();
    my $UIPackage = DE_EPAGES::Order::UI::Basket->new(Servlet=>$Servlet);

    if (! $Basket->get('IsAddressOK')) {
        # verify addresses of basket, collect form errors.
        $Servlet->vars('ObjectID', $Basket->id);
        $UIPackage->VerifyAdresses($Servlet);
        if (not scalar @{$Servlet->form->errors->{'Errors'}}) {
            $Basket->set({'IsAddressOK'=>1});
            if ($Shop->get('TaxCalculationAfterAddress') && $Shop->get('TaxModel') == TAX_MODEL_NET) {
                $Basket->container->recalculate();
            }
        }
    }

    # error handling
    if (! $Basket->get('IsAddressOK')) {
        my $AddressClass = LoadClassByAlias('Address');
        my $hAddressDefaults = $UIPackage->defaultValues( $Basket, $AddressClass );
        my %Defaults;
        $Defaults{'Shipping'.$_} = $hAddressDefaults->{$_} foreach keys %$hAddressDefaults;
        $Defaults{'Billing'.$_}  = $hAddressDefaults->{$_} foreach keys %$hAddressDefaults;

        foreach my $Class ($Shop->get('CustomerClass'), LoadClassByAlias('User')) {
            my $hDefaults = $UIPackage->defaultValues( $Basket, $Class );
            $Defaults{$_} = $hDefaults->{$_} foreach keys %$hDefaults;
        }

        # delete following Defaults when the params available
        if( exists $Defaults{'LanguageID'} and $Servlet->params('Language') ) {
            delete $Defaults{'LanguageID'};
        }
        if( exists $Defaults{'LocaleID'} and $Servlet->params('Locale') ) {
            delete $Defaults{'LocaleID'};
        }
        if( exists $Defaults{'CurrencyID'} and $Servlet->params('Currency') ) {
            delete $Defaults{'CurrencyID'};
        }

        delete $Defaults{'ShippingAddress'} if defined $Basket->get('ShippingAddress');
        delete $Defaults{'Class'};
        delete $Defaults{'ClassID'};

        # show basket template if somthing is not correct
        Error('INVALID_FORM', {
            'ViewVars'=> {
                %Defaults
            },
            'ViewAction' => 'ViewMultiCheckoutAddress',
        });

    }


    # update TaxModel, if address was changed
    my $TaxModelChanged = UpdateTaxModel($Basket->container);
    if (defined $TaxModelChanged) {
        $Basket->container->recalculate();
        SetSessionAttribute( $Servlet->intSessionID, 'TaxModelChanged', 1) unless $Shop->get('VATreleased');
    }

    my $TaxChanged;
    my $TaxChangedDigital;

    # test if we have a TaxMethod
    if($Shop->get('TaxMethod')){
        $Shop->get('TaxMethod')->updateContainerTaxArea($Basket->container);
    }else {
        # update tax area, if address changed
        $TaxChanged = $Basket->updateTaxArea();
        $TaxChangedDigital = $Basket->updateTaxAreaDigital();
    }

    $Servlet->params('TaxChanged', 1) if $TaxChanged;
    $Servlet->params('TaxChangedDigital', 1) if $TaxChangedDigital;

    # trigger hook
    TriggerHook('UI_AfterSaveMultiCheckoutAddress', {'Servlet' => $Servlet});

    # do order, if the DoOrderStep value is set to the current process
    $self->CreateOrder($Servlet,'Address');

    return;
}

sub SaveMultiCheckoutShipping {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook for additional test during this action
    TriggerHook('UI_SaveMultiCheckoutShipping', {'Servlet' => $Servlet});

    # add save and test functions here


    # save shipping by using the basket save functions
    $self->SUPER::Save($Servlet);


    # trigger hook
    TriggerHook('UI_AfterSaveMultiCheckoutShipping', {'Servlet' => $Servlet});

    # do order, if the DoOrderStep value is set to the current process
    $self->CreateOrder($Servlet,'Shipping');

    return;
}

sub SaveMultiCheckoutPayment {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook for additional test during this action
    TriggerHook('UI_SaveMultiCheckoutPayment', {'Servlet' => $Servlet});

    # add save and test functions here

    # save payment by using the basket save functions
    $self->SUPER::Save($Servlet);


    # trigger hook
    TriggerHook('UI_AfterSaveMultiCheckoutPayment', {'Servlet' => $Servlet});

    # do order, if the DoOrderStep value is set to the current process
    $self->CreateOrder($Servlet,'Payment');

    return;
}

sub SaveMultiCheckoutHirePurchase {
    my $self = shift;
    my ($Servlet) = @_;

    # trigger hook for additional test during this action
    TriggerHook('UI_SaveMultiCheckoutHirePurchase', {'Servlet' => $Servlet});

    # add save and test functions here


    # trigger hook
    TriggerHook('UI_AfterSaveMultiCheckoutHirePurchase', {'Servlet' => $Servlet});

    # do order, if the DoOrderStep value is set to the current process
    $self->CreateOrder($Servlet,'HirePurchase');

    return;
}

sub SaveMultiCheckoutConfirmation {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;

    my $Container = $Basket->get('LineItemContainer');
    my $aLineItems = $Container->get('LineItems');
    foreach my $LineItem ( @{$aLineItems} ) {
        next if (! $LineItem->instanceOf('LineItemProduct'));
        my $WantedQuantity = $LineItem->get('Quantity');
        my $Product = $LineItem->get('Product');
        my $LineItemsQuantity = $Product->getQuantityInOtherLineItems($Container, $LineItem);
        my $NewQuantity = $Product->availableQuantity($WantedQuantity, $LineItemsQuantity);
        $LineItem->set({'IsChanged'  => 1, 'Quantity' => $NewQuantity}) if( Compare('Float', $NewQuantity, $WantedQuantity) );
    }

    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'ViewObjectID' => $Basket->id,
            'OrderGUID' => $Basket->get('OrderGUID'),
        },
        'ViewAction' => 'ViewBlocked',
    }) if defined $Basket->get('OrderGUID');


    # trigger hook for additional test during this action
    TriggerHook('UI_SaveMultiCheckoutConfirmation', {'Servlet' => $Servlet});

    # add save and test functions here


    # set the CustomerComment
    my $Form = $Servlet->form->form($Basket, 'Save');
    $Basket->set({'CustomerComment' => $Form->{'CustomerComment'}}) if defined $Servlet->form->value('CustomerComment');

    # trigger hook
    TriggerHook('UI_AfterSaveMultiCheckoutConfirmation', {'Servlet' => $Servlet});

    unless ($Basket->get('IsTacOK') and $Basket->get('IsCustomerInformationOK')) {
        # TAC or Withdrawal not accepted
        Error('INVALID_FORM', {
            'ViewAction' => 'ViewMultiCheckoutConfirmation',
        });
    }

    # do order, if the DoOrderStep value is set to the current process
    $self->CreateOrder($Servlet,'Confirmation');

    return;
}

#==============================================================================
# ViewActions for order process
#==============================================================================

sub ViewMultiCheckoutBasket {
    my $self = shift;
    my ($Servlet) = @_;

    # order process steps are created and hook for changing the process is triggered
    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess',$BasketOrderProcess->tleHash);

    # EPG-28854:
    # Check if basket is empty.
    # DE_EPAGES::Order::API::Object::BasketOrderProcess->new set ViewAction to View for old checkout process.
    # Set correct ViewAction.
    my $ViewEmptyBasket = $Servlet->params('EmptyBasket');
    $Servlet->vars('ViewAction','ViewMultiCheckoutBasket') if defined $ViewEmptyBasket and $ViewEmptyBasket;

    # trigger hook
    TriggerHook('UI_ViewMultiCheckoutBasket', {'Servlet' => $Servlet});

    return;
}

sub ViewMultiCheckoutAddress {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess',$BasketOrderProcess->tleHash);

    my $Basket = $Servlet->object;
    my $User = $Basket->get('User');
    if ($User->get('IsAnonymous')) {
        $Servlet->vars('IsHtmlEMailAllowed', $User->get('IsHtmlEMailAllowed'));

        # set addresses from from user if not set from basket
        my $BillingAddress = $Basket->get('BillingAddress') // $User->get('BillingAddress');
        if (defined $BillingAddress) {
            my $hValues = $BillingAddress->exportableAttributes;
            $Servlet->vars($_, $hValues->{$_}) foreach keys %$hValues;
        }

        my $ShippingAddress = $Basket->get('ShippingAddress') // $User->get('ShippingAddress');
        if (defined $ShippingAddress) {
            my $hValues = $ShippingAddress->exportableAttributes;
            $Servlet->vars('Shipping'. $_, $hValues->{$_}) foreach keys %$hValues;
        }
    }

    # trigger hook
    TriggerHook('UI_ViewMultiCheckoutAddress', {'Servlet' => $Servlet});

    return;
}

sub ViewMultiCheckoutShipping {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;

    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'ViewObjectID' => $Basket->id,
            'OrderGUID' => $Basket->get('OrderGUID'),
        },
        'ViewAction' => 'ViewBlocked',
    }) if defined $Basket->get('OrderGUID');

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess',$BasketOrderProcess->tleHash);

    my $LineItemContainer = $Basket->container;
    my $aBasketShippingMethods = $LineItemContainer->get('ShippingMethods');

    # get the prices for all shipping methods and build loop for template
    my $Formatter = $LineItemContainer->formatter;
    my @Prices = map{
        {
            'ShippingMethodID' => $_->id,
            'ShippingMethodPrice' => $Formatter->round( $_->calculateShippingPrice($LineItemContainer) ),
        }
    } @$aBasketShippingMethods;

    $Servlet->vars('ShippingMethodPrices', \@Prices);

    # check if the default shipping method is valid
    CheckSaveShipping($LineItemContainer);

    # trigger hook
    TriggerHook('UI_ViewMultiCheckoutShipping', {'Servlet' => $Servlet});

    return;
}

sub ViewMultiCheckoutPayment {
    my $self = shift;
    my ($Servlet) = @_;

    my $Basket = $Servlet->object;

    Error('INVALID_FORM', {
        'ViewVars'=> {
            'FormError' => 0,
            'IsBlocked' => 1,
            'ViewObjectID' => $Basket->id,
            'OrderGUID' => $Basket->get('OrderGUID'),
        },
        'ViewAction' => 'ViewBlocked',
    }) if defined $Basket->get('OrderGUID');

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess',$BasketOrderProcess->tleHash);

    my $LineItemContainer = $Basket->container;
    my $aBasketPaymentMethods = $LineItemContainer->get('PaymentMethods');

    # get the prices for all payment methods and build loop for template
    my $Formatter = $LineItemContainer->formatter;
    my @Prices = map{
        {
            'PaymentMethodID' => $_->id,
            'PaymentMethodPrice' => $Formatter->round( $_->calculatePaymentPrice($LineItemContainer) ),
        }
    } @$aBasketPaymentMethods;

    $Servlet->vars('PaymentMethodPrices', \@Prices);

    # check if the default payment method is valid
    CheckSavePayment($LineItemContainer);

    # trigger hook
    TriggerHook('UI_ViewMultiCheckoutPayment', {'Servlet' => $Servlet});

    return;
}

sub ViewMultiCheckoutHirePurchase {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess',$BasketOrderProcess->tleHash);

    # trigger hook
    TriggerHook('UI_ViewMultiCheckoutHirePurchase', {'Servlet' => $Servlet});
    return;
}

sub ViewMultiCheckoutConfirmation {
    my $self = shift;
    my ($Servlet) = @_;

    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    $Servlet->vars('BasketOrderProcess',$BasketOrderProcess->tleHash);

    # trigger hook
    TriggerHook('UI_ViewMultiCheckoutConfirmation', {'Servlet' => $Servlet});

    return;
}

#==============================================================================
# helper function, called on each change action for each step
# test, if the current process has to create the order
# creates the order and show the order information page
#==============================================================================
sub CreateOrder {
    my $self = shift;
    my ($Servlet,$StepAlias) = @_;

    # get the Step wich is responsible for creating the order
    my $BasketOrderProcess = DE_EPAGES::Order::API::Object::BasketOrderProcess->new($Servlet);
    return unless defined $BasketOrderProcess->getOrderStep();
    my $DoOrderStepAlias = $BasketOrderProcess->getOrderStep()->alias;

    #test if the current step has to create the order
    if ($DoOrderStepAlias eq $StepAlias) {

        # do we have a step after the order creation
        my $CurrentStep = $BasketOrderProcess->get('CurrentStep');
        my $NewViewActionAlias = defined $CurrentStep ? $CurrentStep->get('ViewAction')->alias() : 'View' ;

        $Servlet->vars('ViewAction',$NewViewActionAlias);
        return $Servlet->doAction('DoOrder') ;
    }
}

1;
