#==============================================================================
# package      DE_EPAGES::Presentation::API::Constants
# state        public
# description  Constants for the Presentation cartridge
#==============================================================================
package DE_EPAGES::Presentation::API::Constants;
use base Exporter;

use strict;
use DE_EPAGES::Core::API::Constants qw (); # PASSWORD_DUMMY
use DE_EPAGES::Core::API::Log qw ( GetLog );

our @EXPORT_OK = qw(
    $PROCESSMONITOR
    RESERVED_ATTRIBUTENAMES
    PASSWORD_DUMMY
    BASIC_JSON_ATTRIBUTES
    DEFAULT_HISTORY_ENTRIES
    MAX_PREVIEW_CHILD_ITEMS
    ALLOW_SESSION_IN_URL
    JQUERY_JSON_ATTRIBUTES
    JQUERY_JSON_IGNORED_CLASSES
    MAX_IMAGE_UPLOAD_SIZE
    UNITY_VIEW_ACTION_EXCEPTION_REGEX
    RECAPTCHA_PUBLIC_KEY
    RECAPTCHA_PRIVATE_KEY
    RECAPTCHA_CALL_TIMEOUT
    RECAPTCHA_CALL_URL
    RECAPTCHA_FORM_ERROR
    RECAPTCHA_TEST_KEY_HOSTNAME
    UNITY_SF_VIEW_ACTION_PREFIX
    IS_LENGTH_CHECK_ACTIVE
    STRING_COVERAGE
    EPAGES6_LOCALIZED_FIELDS_LENGTH
    UNITY_LOCALIZED_FIELDS_LENGTH
    WRITEABLE_PATH_ATTRIBUTES
    ANTISPAM_TIMESTAMP_ISACTIVE
    WRITEABLE_PATH_ATTRIBUTES
    FORMSUBMIT_TIMEFRAME
);

use DE_EPAGES::Core::API::EpagesConfig qw( EpagesConfig );
use DE_EPAGES::Presentation::API::SpamprotectionConfig qw( SpamprotectionConfig );

#========================================================================================
# global       $PROCESSMONITOR | true if the monitor is enabled | boolean
# global       $DEFAULT_HISTORY_ENTRIES | count of history entries (20) | int
#========================================================================================
our $PROCESSMONITOR = 1;
use constant DEFAULT_HISTORY_ENTRIES => 20;

#========================================================================================
# global       RESERVED_ATTRIBUTENAMES | reserved names witch cannot be used as
#                   attribute names | array
#========================================================================================
use constant RESERVED_ATTRIBUTENAMES => qw (ChangeAction CurrencyID Language Locale
    ObjectID ObjectPath Price ShopID Style System Value ViewAction UnityAlias);

#========================================================================================
# global       PASSWORD_DUMMY | the dummy text which is displayed in password fields
#                   (********) | string
#========================================================================================
sub PASSWORD_DUMMY { ## no critic qw(RequireCommentForExportedFunctions)
    return DE_EPAGES::Core::API::Constants::PASSWORD_DUMMY;
}

#========================================================================================
# global       BASIC_JSON_ATTRIBUTES | deprecated only used by dojo json,
#                list of attributes names for JSONChildren
#               | ref.array.string
#========================================================================================
use constant BASIC_JSON_ATTRIBUTES => [qw(ObjectID Alias NameOrAlias ClassID Position WebUrl WebUrlSSL Path)];

#========================================================================================
# global       JQUERY_JSON_ATTRIBUTES | list of attributes names for JSONChildrenJSTree
#               | ref.array.string
#========================================================================================
use constant JQUERY_JSON_ATTRIBUTES => [qw(ClassID WebUrl WebUrlSSL)];

#========================================================================================
# global       JQUERY_JSON_IGNORED_CLASSES | list of ignored classed for JSONChildrenJSTree
#               | ref.array.string
#========================================================================================
use constant JQUERY_JSON_IGNORED_CLASSES => [qw(CustomForm ImageGallery GuestBook Forum BlogPost)];

#========================================================================================
# global       MAX_PREVIEW_CHILD_ITEMS | number of maximal visible child items | int
#========================================================================================
use constant MAX_PREVIEW_CHILD_ITEMS => 1000;


#========================================================================================
# global       MAX_IMAGE_UPLOAD_SIZE | max image size in kB for uploading images | int
#               The value of  MAXCONTENTLENGTH in WebInterface.conf is the max size
#               use this constant to set the limit lower than MAXCONTENTLENGTH
#========================================================================================
use constant MAX_IMAGE_UPLOAD_SIZE => undef;

#========================================================================================
# global       ALLOW_SESSION_IN_URL | Enables session handling for user agents that do
#               do accept cookies for all or some domains. Default=1.
#               This value is configurable in the epages.conf file, section
#               DE_EPAGES::Presentation, key AllowSessionInURL | integer
#========================================================================================
sub ALLOW_SESSION_IN_URL() { ## no critic qw(RequireCommentForExportedFunctions)
    return EpagesConfig()->get( 'DE_EPAGES::Presentation', 'AllowSessionInURL', 1 );
}

#========================================================================================
# global       UNITY_VIEW_ACTION_EXCEPTION_REGEX | regex list not to be prefixed with Unity | int
#========================================================================================
use constant UNITY_VIEW_ACTION_EXCEPTION_REGEX => qr/^(JSON|Unity|DereferredRedirect)/;


#========================================================================================
# global       RECAPTCHA_PUBLIC_KEY | returns ReCaptcha public key
#========================================================================================
sub RECAPTCHA_PUBLIC_KEY() { ## no critic qw(RequireCommentForExportedFunctions)
    return EpagesConfig()->get('GoogleCaptcha', 'ReCaptchaPublicKey');
}

#========================================================================================
# global       RECAPTCHA_PRIVATE_KEY | returns ReCaptcha private key
#========================================================================================
sub RECAPTCHA_PRIVATE_KEY() { ## no critic qw(RequireCommentForExportedFunctions)
    return EpagesConfig()->get('GoogleCaptcha', 'ReCaptchaPrivateKey');
}

#========================================================================================
# global       RECAPTCHA_CALL_TIMEOUT | Timeout for Google ReCaptcha API | int
#========================================================================================
use constant RECAPTCHA_CALL_TIMEOUT => 20;

#========================================================================================
# global       RECAPTCHA_CALL_URL | Google url | String
#========================================================================================
use constant RECAPTCHA_CALL_URL => 'https://www.google.com';

#========================================================================================
# global       RECAPTCHA_TEST_KEY_HOSTNAME | Google test key hostname response| String
#========================================================================================
use constant RECAPTCHA_TEST_KEY_HOSTNAME => 'testkey.google.com';

#========================================================================================
# global       RECAPTCHA_FORM_ERROR | Form error for Google ReCaptcha | hash
#========================================================================================
use constant RECAPTCHA_FORM_ERROR => {
        'Reason'=> 'RECAPTCHA_WRONG',
        'Form'  => 'CustomForm',
        'Name'  => 'Recaptcha',
};

#========================================================================================
# global       UNITY_SF_VIEW_ACTION_PREFIX | Prefix for SF view actions of an Unity shop
#                | string
#========================================================================================
use constant UNITY_SF_VIEW_ACTION_PREFIX => 'UnityView';


#==============================================================================
# function     IS_LENGTH_CHECK_ACTIVE
# syntax       IS_LENGTH_CHECK_ACTIVE();
# description  Returns a flag from epages.conf which tells whether length check is active
# return       boolean. 1 if active, 0 otherwise
#==============================================================================
sub IS_LENGTH_CHECK_ACTIVE {
    my $isActive = int(EpagesConfig()->get('LocalizedFields', 'isActive'));
    if ($isActive < 0 || $isActive > 1) {
        GetLog()->warn("Wrong isActive configuration: $isActive. isActive must be in [0, 1] range. reset to 0");
        $isActive = 0;
    }
    return $isActive;
};

#==============================================================================
# function     STRING_COVERAGE
# syntax       STRING_COVERAGE();
# description  Returns a float from epages.conf which tells what percent of string
#               should match in the comparison to be considered equal
# return       float. from 0 to 1. 1 means 100 % coverage, just like in normal comparison
#==============================================================================
sub STRING_COVERAGE {
    my $coverage = 0 + EpagesConfig()->get('LocalizedFields', 'coverage');
    if ($coverage < 0 || $coverage > 1) {
        GetLog()->warn("Wrong coverage configuration: $coverage. Coverage must be in [0, 1] range. reset to 0");
        $coverage = 0;
    }
    return $coverage;
};

#==============================================================================
# function     EPAGES6_LOCALIZED_FIELDS_LENGTH
# syntax       EPAGES6_LOCALIZED_FIELDS_LENGTH()->{'Description'};
# description  Returns a hash of field lengths of localized strings for ep6
# return       hash of ints by field name
#==============================================================================
sub EPAGES6_LOCALIZED_FIELDS_LENGTH {
    my $Epages6MaxLength = int(EpagesConfig()->get('LocalizedFields', 'Epages6LocalizedFieldsMaxLength'));
    if ($Epages6MaxLength < 0) {
        GetLog()->warn("Wrong Epages6MaxLength configuration: $Epages6MaxLength. Epages6MaxLength must be a positive integer. reset to 0");
        $Epages6MaxLength = 0;
    }
    return {
        AcceptCustomerInformationText    => $Epages6MaxLength,
        AcceptTacText                    => $Epages6MaxLength,
        Address                          => $Epages6MaxLength,
        AvailabilityComment              => $Epages6MaxLength,
        BasketBelowText                  => $Epages6MaxLength,
        BasketOverText                   => $Epages6MaxLength,
        Description                      => $Epages6MaxLength,
        Comment                          => $Epages6MaxLength,
        MainCharacteristics              => $Epages6MaxLength,
        Note                             => $Epages6MaxLength,
        OrderConfirmationText            => $Epages6MaxLength,
        OrderProcessAddressText          => $Epages6MaxLength,
        OrderProcessBasketText           => $Epages6MaxLength,
        OrderProcessCheckText            => $Epages6MaxLength,
        OrderProcessHeaderText           => $Epages6MaxLength,
        OrderProcessPaymentText          => $Epages6MaxLength,
        OrderProcessShippingText         => $Epages6MaxLength,
        PriceTagInfoMultiProductGross    => $Epages6MaxLength,
        PriceTagInfoMultiProductNet      => $Epages6MaxLength,
        PriceTagInfoMultiProductTaxFree  => $Epages6MaxLength,
        PriceTagInfoSingleProductGross   => $Epages6MaxLength,
        PriceTagInfoSingleProductNet     => $Epages6MaxLength,
        PriceTagInfoSingleProductTaxFree => $Epages6MaxLength,
        ReferencePriceText               => $Epages6MaxLength,
        Slogan                           => $Epages6MaxLength,
        Text                             => $Epages6MaxLength,
        Text1                            => $Epages6MaxLength,
        Text2                            => $Epages6MaxLength,
        Text3                            => $Epages6MaxLength,
    };
}

#==============================================================================
# function     UNITY_LOCALIZED_FIELDS_LENGTH
# syntax       UNITY_LOCALIZED_FIELDS_LENGTH()->{'Description'};
# description  Returns a hash of field lengths of localized strings for unity
# return       hash of ints by field name
#==============================================================================
sub UNITY_LOCALIZED_FIELDS_LENGTH {
    my $UnityMaxLength = int(EpagesConfig()->get('LocalizedFields', 'UnityLocalizedFieldsMaxLength'));
    if ($UnityMaxLength < 0) {
        GetLog()->warn("Wrong UnityMaxLength configuration: $UnityMaxLength. UnityMaxLength must be a positive integer. reset to 0");
        $UnityMaxLength = 0;
    }
    return {
        AcceptCustomerInformationText  => $UnityMaxLength,
        AcceptTacText                  => $UnityMaxLength,
        Address                        => $UnityMaxLength,
        AvailabilityComment            => $UnityMaxLength,
        BasketBelowText                => $UnityMaxLength,
        Description                    => $UnityMaxLength,
        Header                         => $UnityMaxLength,
        MainCharacteristics            => $UnityMaxLength,
        Note                           => $UnityMaxLength,
        OrderDocumentsCreditNoteText1  => $UnityMaxLength,
        OrderDocumentsFooter           => $UnityMaxLength,
        OrderDocumentsInvoiceText1     => $UnityMaxLength,
        OrderDocumentsPackingSlipText1 => $UnityMaxLength,
        OrderProcessAddressText        => $UnityMaxLength,
        OrderProcessCheckText          => $UnityMaxLength,
        OrderProcessPaymentText        => $UnityMaxLength,
        OrderProcessShippingText       => $UnityMaxLength,
        Signature                      => $UnityMaxLength,

    };
}

#========================================================================================
# global       WRITEABLE_PATH_ATTRIBUTES | List of attributes that are writable and
#               that are used to construct filenames. Must be checked because of potential
#               path traversal attacks | string
#========================================================================================
use constant WRITEABLE_PATH_ATTRIBUTES => [qw( FilePathPart UnityThemeStylePath )];

#========================================================================================
# function       FORMSUBMIT_TIMEFRAME
# state          public
#----------------------------------------------------------------------------------------
# syntax         my $FormsubmitConfigValues = FORMSUBMIT_TIMEFRAME();
#----------------------------------------------------------------------------------------
# description    Gets feature configuration from configuration file
#----------------------------------------------------------------------------------------
# return         ref.hash.boolean
#                 <ul>
#                       <li> Feature_IsActive           | boolean </>
#                       <li> Feature_TestMode           | boolean </>
#                 </ul>
#========================================================================================
sub FORMSUBMIT_TIMEFRAME() { ## no critic qw(RequireCommentForExportedFunctions)
    return {
            'Feature_IsActive'          => SpamprotectionConfig()->get( 'DE_EPAGES::Presentation', 'Feature_IsActive'),
            'Feature_TestMode'          => SpamprotectionConfig()->get( 'DE_EPAGES::Presentation', 'Feature_TestMode')
    };
}
1;
