#========================================================================================
# package      DE_EPAGES::Product::API::Object::ProductType
# state        public
# base         DE_EPAGES::Object::API::Object::Class
# description  object interface for ProductType.
#========================================================================================
package DE_EPAGES::Product::API::Object::ProductType;
use base DE_EPAGES::Object::API::Object::Class;

use strict;

use DE_EPAGES::Core::API::File qw ( CopyFile );
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Trigger::API::Trigger qw ( TriggerHook );
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias LoadRootObject );
use DE_EPAGES::Product::API::Table::ProductTypeFontColor qw ( DeleteProductTypeFontColor GetPKeyProductTypeFontColorsByProductTypeID GetInfoProductTypeFontColorsByProductTypeID );
use DE_EPAGES::Product::API::Table::ProductTypeCustomFont qw ( DeleteProductTypeCustomFont GetPKeyProductTypeCustomFontsByProductTypeID GetInfoProductTypeCustomFontsByProductTypeID );
use DE_EPAGES::Product::API::Table::ProductTypeFileType qw ( DeleteProductTypeFileType GetPKeyProductTypeFileTypesByProductTypeID ExistsProductTypeFileType);
use DE_EPAGES::Product::API::Table::ProductTypeSystemFont qw ( DeleteProductTypeSystemFont GetPKeyProductTypeSystemFontsByProductTypeID ExistsProductTypeSystemFont);
use DE_EPAGES::Product::API::XML::ProductTypeDeleteHandler; ##no critic (RequireCorrectImports)
use DE_EPAGES::Product::API::XML::ProductTypeExportDriver;  ##no critic (RequireCorrectImports)
use DE_EPAGES::Product::API::XML::ProductTypeImportHandler; ##no critic (RequireCorrectImports)

#ERRORDEFINITION
sub ErrorDefinition {
    return {
        'MissingBaseClass' => 'Product Type with ID #ObjectID has an empty BaseClass attribute. '
            . 'Deleting this object could result in the deletion of all products in this database!',
    }
}

#========================================================================================
# function     className
# state        public
#----------------------------------------------------------------------------------------
# syntax       $ClassName = $ProductType->className;
#----------------------------------------------------------------------------------------
# description  Returns the class name, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $ClassName | class name | String
#========================================================================================
sub className  { 'ProductType' }

#========================================================================================
# function     dbPackage
# state        public
#----------------------------------------------------------------------------------------
# syntax       $DbPackage = $ProductType->dbPackage;
#----------------------------------------------------------------------------------------
# description  Returns the database package, used for DAL access.
#----------------------------------------------------------------------------------------
# return       $DbPackage | database package name | String
#========================================================================================
sub dbPackage { __PACKAGE__ }

#========================================================================================
# function     insert
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ProductType = DE_EPAGES::Product::API::Object::ProductType->insert($hInfo);
# example      my $Class = LoadClassByAlias('ProductType');
#               my $ProductType = $Class->insertObject($hInfo);
#----------------------------------------------------------------------------------------
# description  Insert this info as object, afterwards the hook 'OBJ_InsertProductType' is triggered.
#----------------------------------------------------------------------------------------
# input        $hInfo | attributes of new object | ref.hash
# return       $ProductType | new object | object
# hook         OBJ_InsertProductType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProductTypeID - class id             - int
#               </ul>
#========================================================================================
sub insert {
    my $class = shift;
    my ($hInfo) = @_;

    # increment the usage counter of the ProductType feature
    my $Shop = $hInfo->{'Shop'} || $hInfo->{Parent}->getSite();
    $class->incrementFeature($Shop) unless $hInfo->{'Alias'} eq 'ProductClass';

    my $self = $class->SUPER::insert($hInfo);
    TriggerHook('OBJ_InsertProductType', {'Object' => $self, 'ProductTypeID' => $self->id });
    delete $hInfo->{$_} foreach qw ( CustomizeIsFontSizeSelectable CustomizeFontSize ); ## no critic qw(ProhibitLongLines)
    return $self;
}

#========================================================================================
# function     incrementFeature
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->incrementFeature($Shop)
#----------------------------------------------------------------------------------------
# description  Increments ProductType feature for a shop. Called from insert()
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
#========================================================================================
sub incrementFeature {
    my $self = shift;
    my ($Shop) = @_;

    $Shop->featureIncrementValue( 'ProductTypes', 1 );
    return;
}

#========================================================================================
# function     delete
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ProductType->delete;
#----------------------------------------------------------------------------------------
# description  Deletes this object, therefore the hook 'OBJ_DeleteProductType' is triggered.
#----------------------------------------------------------------------------------------
# hook         OBJ_DeleteProductType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProductTypeID - class id             - int
#               </ul>
#========================================================================================
sub delete {
    my $self = shift;

    # forbid deletion of a product type without base class to avoid deleting
    # all products in the database (UNTIY-6249)
    Error('MissingBaseClass', { 'ObjectID' => $self->id }) unless defined $self->baseClass;

    # increment the usage counter of the ProductType feature
    my $Shop = $self->getSite;
    my $Alias = $self->alias;

    TriggerHook('OBJ_DeleteProductType', {'Object' => $self, 'ProductTypeID' => $self->id });
    DeleteProductTypeCustomFont($_) foreach @{GetPKeyProductTypeCustomFontsByProductTypeID($self->id)};
    DeleteProductTypeFileType($_) foreach @{GetPKeyProductTypeFileTypesByProductTypeID($self->id)};
    DeleteProductTypeFontColor($_) foreach @{GetPKeyProductTypeFontColorsByProductTypeID($self->id)};
    DeleteProductTypeSystemFont($_) foreach @{GetPKeyProductTypeSystemFontsByProductTypeID($self->id)};
    $self->SUPER::delete;
    $self->decrementFeature( $Shop ) unless $Alias eq 'ProductClass';
    return;
}

#========================================================================================
# function     decrementFeature
# state        private
#----------------------------------------------------------------------------------------
# syntax       $self->decrementFeature( $Shop )
#----------------------------------------------------------------------------------------
# description  Decrements ProductType feature for a shop. Called from delete()
#----------------------------------------------------------------------------------------
# input        $Shop | shop object | DE_EPAGES::Shop::API::Object::Shop
#========================================================================================
sub decrementFeature {
    my $self = shift;
    my ($Shop) = @_;

    $Shop->featureIncrementValue( 'ProductTypes', -1 );
    return;
}

#========================================================================================
# function     set
# state        private
#----------------------------------------------------------------------------------------
# syntax       $ProductType->set($hValues, $LanguageID);
#----------------------------------------------------------------------------------------
# description  This function triggers hooks 'OBJ_BeforeUpdateProductType' and 'OBJ_AfterUpdateProductType'.
#----------------------------------------------------------------------------------------
# input        $hValues | attribute name => value hash | ref.hash
# input        $LanguageID | (optional; required for localized attributes) language id | int
#----------------------------------------------------------------------------------------
# hook         OBJ_BeforeUpdateProductType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProductTypeID - class id             - int
#                 <li>Values        - new values for object - ref.hash
#                 <li>LanguageID    - values of this language will be updated - int
#               </ul>
# hook         OBJ_AfterUpdateProductType | hook parameter keys :
#               <ul>
#                 <li>Object        - this object          - object
#                 <li>ProductTypeID - class id             - int
#                 <li>Values        - updated values       - ref.hash
#                 <li>LanguageID    - values updated of this language - int
#               </ul>
#========================================================================================
sub set {
    my $self = shift;
    my ($hValues, $LanguageID) = @_;

    TriggerHook('OBJ_BeforeUpdateProductType', {'Object'=>$self, 'Values'=> $hValues, 'ProductTypeID' => $self->id, 'LanguageID' => $LanguageID});
    $self->SUPER::set($hValues, $LanguageID);
    TriggerHook('OBJ_AfterUpdateProductType',  {'Object'=>$self, 'Values'=> $hValues, 'ProductTypeID' => $self->id, 'LanguageID' => $LanguageID});
    return;
}

sub importXML {
    my ($ObjectPackage, $Handler, $hElement, $Class, $hAttributes) = @_;

    my $Parent = $Handler->topID('Object');
    my $Shop = $Parent->getSite();

    if (not $Handler->topID('IsReference')) {

        if( $Parent->id == $Shop->id && $hAttributes->{Alias} eq 'ProductClass' ) {
            $hAttributes->{BaseClass} = LoadClassByAlias('Product');
        }
        else {
            $hAttributes->{BaseClass} = $Shop->get('ProductClass');
        }

        $hAttributes->{Package} = 'DE_EPAGES::Product::API::Object::Product'
            unless exists $hAttributes->{Package};
    }

    return $ObjectPackage->SUPER::importXML($Handler, $hElement, $Class, $hAttributes);
}

#========================================================================================
# function     exportXMLAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hAttributes = $Object->exportXMLAttributes( $Driver, $hAllAttributes );
#----------------------------------------------------------------------------------------
# description  Returns plain attributes of object to XML export driver. This function
#               removes bit attributes with value 0.
#----------------------------------------------------------------------------------------
# input        $Driver | xml export driver with support function addObject | object
# input        $hAllAttributes | all attributes of object,
#                                 see $Object-><function exportableAttributes> | ref.hash
# return       $hAttributes | plain attributes | ref.hash.string
#========================================================================================
sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $hAttributes) = @_;
    $hAttributes = $self->SUPER::exportXMLAttributes($Driver, $hAttributes);

    return $hAttributes;
}

sub exportXMLElements {
    my $self = shift;
    my ($Driver, $allAttributes) = @_;
    $self->SUPER::exportXMLElements($Driver, $allAttributes);
    if ($Driver->exportLevel == $self->class->get('ExportLevel')) {
        DE_EPAGES::Object::API::XML::ObjectExportDriver::ProductTypeCustomFontsByProductType($Driver);
        DE_EPAGES::Object::API::XML::ObjectExportDriver::ProductTypeFileTypesByProductType($Driver);
        DE_EPAGES::Object::API::XML::ObjectExportDriver::ProductTypeFontColorsByProductType($Driver);
        DE_EPAGES::Object::API::XML::ObjectExportDriver::ProductTypeSystemFontsByProductType($Driver);

        #my $PaymentLevelClass = LoadClassByAlias('PaymentLevel');
        #my $aPaymentLevels = $PaymentLevelClass->getObjects('ByPaymentMethodID', {'PaymentMethodID'=>$self->id});
        #$Driver->addObject( 'PaymentLevel', $_ ) foreach @$aPaymentLevels;

    }



    return;
}



#========================================================================================
# function     canInvalidatePageCache
# state        public
#----------------------------------------------------------------------------------------
# syntax       $CanInvalidate = $Object->canInvalidatePageCache;
#----------------------------------------------------------------------------------------
# description  Overwrites method DE_EPAGES::Object::API::Object::Object->canInvalidatePageCache
#               to invalidate page cache.
#----------------------------------------------------------------------------------------
# return       $CanInvalidate | always true | boolean
#========================================================================================
sub canInvalidatePageCache {
    return 1;
}

#========================================================================================
# function     default
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = DE_EPAGES::Product::API::Object::ProductType->default( $Parent, $hInfo );
# example      $hDefaults = $Class->defaultAttributes($Parent, $hInfo); # for insert a new object
#               $hDefaults = $Object->class->defaultAttributes($Object->parent, $hValues); # for an existing object
#----------------------------------------------------------------------------------------
# description  Returns default attribute values for new objects.
#               Used by <function DE_EPAGES::Object::Object::Class::defaultAttributes>
#----------------------------------------------------------------------------------------
# input        $Parent | parent object of the new object | object
# input        $hInfo | attribute name => value hash (passed to insert()) | ref.hash
# return       $hValues | hash with default attribute values (name => value) | ref.hash
#========================================================================================
sub default {
    my $class = shift;
    my ($Parent, $hInfo) = @_;
    return {
        %{$class->SUPER::default($Parent, $hInfo)},
        'CustomizeIsFontSizeSelectable' => 0,
        'CustomizeFontSize' => 12,
    };
}


sub existsSystemFont{
    my $self = shift;
    my ($SystemFontOBJ) = @_;
    my $aSystemFonts = GetPKeyProductTypeSystemFontsByProductTypeID($self->id);
    my $aCustomFonts = GetPKeyProductTypeCustomFontsByProductTypeID($self->id);
    if((scalar(@$aSystemFonts) == 0) && (scalar(@$aCustomFonts) == 0)){
        my $default = {'Arial' => 1};

        # copy js file for font (IE Hack) from Storetypes to directory of ProductType
        $self->copyIESystemFontFile(LoadRootObject()->folder('SystemFonts')->child('Arial'));

        return defined $default->{$SystemFontOBJ->alias};
    }
        else {
    return ExistsProductTypeSystemFont($self->id, $SystemFontOBJ->id);
    }
}

sub existsFileType{
    my $self = shift;
    my ($FileTypeOBJ) = @_;
    my $aFileTypes = GetPKeyProductTypeFileTypesByProductTypeID($self->id);
    if(scalar(@$aFileTypes) == 0){
        my $default = {'jpg' => 1, 'png' => 1, 'gif' => 1};
        return defined $default->{$FileTypeOBJ->alias};
    } else {
        return ExistsProductTypeFileType($self->id, $FileTypeOBJ->id);
    }
}

sub getInfoSystemFonts{
    my $self = shift;

    my $aAllSystemFonts = LoadClassByAlias('SystemFont')->objects();
    my $aSelectedSystemFonts =  [ map { $self->existsSystemFont($_) ? $_ : () } @$aAllSystemFonts ] ;
    return $aSelectedSystemFonts;
}

sub getInfoFileTypes{
    my $self = shift;

    my $aAllFileTypes = LoadClassByAlias('FileType')->objects();
    my $aSelectedFileTypes =  [ map { $self->existsFileType($_) ? $_ : () } @$aAllFileTypes ] ;
    return $aSelectedFileTypes;

}

sub getInfoFontColors{
    my $self = shift;
    my $aFontColors = GetInfoProductTypeFontColorsByProductTypeID($self->id);
    if(scalar(@$aFontColors) == 0){
        return [{'ColorValue' => 'black','Position' => 0}];
    } else {
        return $aFontColors;
    }
}

sub _BaseFontFileName{
    my ($FileName) = @_;
    my $FileNameWithoutExtension = $FileName;
    $FileNameWithoutExtension =~ s/\.[^\.]*$//g;
    return $FileNameWithoutExtension;
}

sub getInfoCustomFonts{
    my $self = shift;
    my $ahAllCustomFonts  = GetInfoProductTypeCustomFontsByProductTypeID($self->id);
    return [
        map { { %$_, 'BaseFontFileName' => _BaseFontFileName($_->{'FileName'}) } } @$ahAllCustomFonts
    ];
}

sub copyIESystemFontFile {
    my $self = shift;
    my ($SystemFont) = @_;

    # copy js file for font (IE Hack) from Storetypes to directory of ProductType
    my $FileName = $SystemFont->alias;
    $FileName =~ s/\s+//g;
    $FileName .= '.js';

    my $SourceFile = join '/', LoadRootObject()->get('PublicStoreTypePath'), 'IESystemFonts', $FileName;
    my $TargetFile = join '/', $self->get('PublicPath'), $FileName;
    CopyFile( $SourceFile, $TargetFile, { 'create_directory' => 1 } );
}

1;
