#========================================================================================
# package      DE_EPAGES::Tax::API::Attributes::Shop
# state        private
# description  Attribute interface for Shop.
#========================================================================================
package DE_EPAGES::Tax::API::Attributes::Shop;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use DE_EPAGES::Core::API::Error qw ( Error );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::Object::API::Factory qw (LoadRootObject GetRootObjectID Transaction );
use DE_EPAGES::Object::API::Attributes::Reference;
use DE_EPAGES::Tax::API::Shop qw ( GetTaxMethodsFolder );

my $REFERENCE_PACKAGE = 'DE_EPAGES::Object::API::Attributes::Reference';

sub ErrorDefinition {
    return {
            'TaxMatrixNotCustomized' => 'TaxMatrix not customized. Shop: #ShopAlias, TaxMatrix: #TaxMatrixPath, Store: #Store',
    };
}

sub getAttributes {
    shift;
    my ($Object, $aNames) = @_;
    my $System = LoadRootObject();
    my $Shop = $Object->getSite;
    my $hInfo = {};
    foreach my $Name (@$aNames) {
        if ($Name eq 'TaxMatrixID') {
            my $TaxMatrixID = $REFERENCE_PACKAGE->getAttribute($Object, $Name);
            $hInfo->{$Name} = $TaxMatrixID;
        }
        elsif (($Name eq 'TaxMatrixDigitalID') and $System->get('TaxMatrixDigital') and not $Shop->get('IsUnityRegistered')) {
            # If TaxMatrixDigital is undefined, set it to the system's TaxMatrixDigital
            my $TaxMatrixDigitalID = $REFERENCE_PACKAGE->getAttribute($Object, $Name);
            unless (defined $TaxMatrixDigitalID) {
                my $SystemTaxMatrixDigitalID = $System->get('TaxMatrixDigital')->id;
                $REFERENCE_PACKAGE->setAttribute($Object, $Name, $SystemTaxMatrixDigitalID );
                $TaxMatrixDigitalID = $SystemTaxMatrixDigitalID;
            }
            $hInfo->{$Name} = $TaxMatrixDigitalID;
        }
        elsif (($Name eq 'DefaultTaxAreaDigitalID') and $System->get('TaxMatrixDigital') and not $Shop->get('IsUnityRegistered')) {
            # If DefaultTaxAreaDigital is undefined, set it to the country of the shop address
            my $DefaultTaxAreaDigitalID = $REFERENCE_PACKAGE->getAttribute($Object, $Name);
            unless (defined $DefaultTaxAreaDigitalID) {
                my $ShopCountryID = $Shop->get('Address')->get('CountryID');
                if (defined $ShopCountryID) {
                    my $DefaultTaxAreaDigital = $Shop->get('TaxMatrixDigital')->taxAreaDigitalByCountry($ShopCountryID);
                    if ($DefaultTaxAreaDigital){
                        $REFERENCE_PACKAGE->setAttribute($Object, $Name, $DefaultTaxAreaDigital->id);
                    }
                }
            }
            $hInfo->{$Name} = $DefaultTaxAreaDigitalID;
        }
        elsif (($Name eq 'DefaultTaxClassDigitalID') and $System->get('TaxMatrixDigital') and not $Shop->get('IsUnityRegistered')) {
            # If DefaultTaxClassDigital is undefined, set it to the system's DefaultTaxClassDigital
            my $DefaultTaxClassDigitalID = $REFERENCE_PACKAGE->getAttribute($Object, $Name);
            unless (defined $DefaultTaxClassDigitalID) {
                $DefaultTaxClassDigitalID = $System->get('TaxMatrixDigital')->get('DefaultTaxClassDigital')->id;
                $REFERENCE_PACKAGE->setAttribute($Object, $Name, $DefaultTaxClassDigitalID );
            }
            $hInfo->{$Name} = $DefaultTaxClassDigitalID;
        }
        elsif ($Name eq 'TaxMethods') {
            $hInfo->{$Name} = GetTaxMethodsFolder($Object)->children;
        }
        elsif ($Name eq 'TaxMethodID') {
            my $TaxMethodID = $REFERENCE_PACKAGE->getAttribute($Object, $Name);
            $hInfo->{$Name} = $TaxMethodID;
        }
    }
    return $hInfo;
}

sub setAttributes {
    shift;
    my ($Object, $hUpdate) = @_;
    foreach my $Name (keys %$hUpdate) {
        if ($Name eq 'TaxMatrixID') {
            $REFERENCE_PACKAGE->setAttribute($Object, $Name, $hUpdate->{$Name});
        }
        elsif ($Name eq 'TaxMatrixDigitalID') {
            $REFERENCE_PACKAGE->setAttribute($Object, $Name, $hUpdate->{$Name});
        }
        elsif ($Name eq 'DefaultTaxAreaDigitalID') {
            $REFERENCE_PACKAGE->setAttribute($Object, $Name, $hUpdate->{$Name});
        }
        elsif ($Name eq 'DefaultTaxClassDigitalID') {
            $REFERENCE_PACKAGE->setAttribute($Object, $Name, $hUpdate->{$Name});
        }
        elsif ($Name eq 'TaxMethodID') {
            Transaction(
                sub {
                    $REFERENCE_PACKAGE->setAttribute($Object, $Name, $hUpdate->{$Name});
                    my $TaxMatrix = $Object->get('TaxMatrix');
                    if (defined $TaxMatrix) {
                        if ($TaxMatrix->instanceOf('TaxMatrix') and $TaxMatrix->parent->id != $Object->id and defined $hUpdate->{$Name}) {
                            Error(
                                'TaxMatrixNotCustomized', {
                                    'Shop'          => $Object,
                                    'ShopAlias'     => $Object->alias,
                                    'TaxMatrixPath' => $TaxMatrix->get('Path'),
                                    'Store'         => GetCurrentDBHandle()->cacheKey,
                                });
                        }
                        if (!defined $hUpdate->{$Name}) {
                            $TaxMatrix->set({'LineItemClass' => undef});
                        }
                    }
                });
        }
    }
    return;
}


sub deleteObject {
    shift;
    my ($Object) = @_;
    $REFERENCE_PACKAGE->deleteObject($Object);
}

sub defaultAttributes {
    return {
        'TaxMatrix'=> LoadRootObject->get('TaxMatrix'),
        'TaxMatrixDigital'=> LoadRootObject->get('TaxMatrixDigital'),
    };
}

sub exportXMLAttributes {
    my $self = shift;
    my ($Driver, $Object, $aNames, $hAttributes) = @_;
    # for import set matrix as
    if (exists $hAttributes->{'TaxMatrix'}) {
        my $Matrix = $Object->get('TaxMatrix');
        delete $hAttributes->{'TaxMatrix'} if $Matrix->getSite()->id != GetRootObjectID();
    }
    return;
}

1;
